#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
U2IFJupyter.py
uGao u2dplot I/F on Jupyter and on the dark mode
"""
from __future__ import print_function
import os
import Manyo
import Manyo.MLF as mm
import numpy as np
import datetime

from uGao.u2dplotBase import U2Params, D2Chart, DataConverter, format_int_range
from uGao.uGaoColorMap import COLOR_MAPS
from uGao.uGaoUtil import IFEvtProp, MASKVALUE

import matplotlib
ISJUPYTER = False
if matplotlib.get_backend() == "inline":
    # print("U2IFJupyter : backend = inline")
    ISJUPYTER = True
    import matplotlib.pyplot as plt
else:
    # print("U2IFJupyter : backend is not inline")
    from matplotlib.figure import Figure
    from matplotlib.backends.backend_agg import FigureCanvasAgg as FigCanvas

#########################################
# U2IFJupyter
#########################################
class U2IFJupyter(object):
    """
    U2IFJupyter class

    This enables to plot on Jupyter and to save plot file using u2dplot of uGao

    import uGao.U2IFJupyter as mp
    p = mp.U2IFJupyter(sizeX=10.0, sizeY=5.0)
    p.SetKeys("index", "index", "Intensity")
    p.SetData(dat)
    p.SetRange(False, 0.0, 40.0)
    p.SetTitles("Test in Dark", "")
    p.SetLabels("QA", "hw")
    p.SaveFile("test_InDark.png")
    # On Jupyter
    p.Show()
    """

    #########################################
    def __init__(self, data=None, orderNo=1, sizeX=7.0, sizeY=7.0, dpi=100):
        """
        Constructor
        @param data(list of numpy arrays or ElementContainerMatrix or -Array)
        @param orderNo(int) Plot number
        @param sizeX (float) size of X axis direction
        @param sizeY (float) size of Y axis direction
        @param dpi (float) dot per inch
        @retval None
        """
        if ISJUPYTER:
            # print("U2IFJupyter : ISJUPYTER is True")
            self.plt = plt
            self.fig = plt.figure(figsize=(sizeX, sizeY))
            self.canvas = self.fig.canvas
        else:
            # print("U2IFJupyter : ISJUPYTER is False")
            self.fig = Figure((sizeX, sizeY), dpi)
            self.canvas = FigCanvas(self.fig)
        self.order = orderNo
        self.params = U2Params()
        self.sliceMode = False
        self.ifi = IFEvtProp(orderNo)
        self.ifi.AddProperty('params', lambda: self.params)
        self.ifi.AddProperty('slice_mode', lambda: self.sliceMode)

        self.d2 = None

        self.ndata = None
        self.data = data
        self.keys = (None, None, None)
        self.intRange = None
        self.converter = DataConverter()

    ###################################################
    def Show(self):
        """
        """
        if ISJUPYTER:
            if self.ndata is None:
                if not self.SetData():
                    msg = "U2IFJupyter::SaveFile >> data is invalid."
                    raise UserWarning(msg)

            if self.d2 is not None:
                del self.d2

            self.d2 = D2Chart(self, self.canvas, self.ndata, True)

            self.plt.show()
        else:
            pass

    #########################################
    def SetKeys(self, xkey, ykey, intkey=None, rkey=None):
        """キー文字列設定処理

        Parameters:
            xkey (str or None): X Key
            ykey (str or None): Y Key
            intkey (str or None): Int Key
            rkey (str or None): Range Key
        """
        if xkey is not None and not isinstance(xkey, str):
            raise PlotException('Common', 'C025', ('xkey',))
        if ykey is not None and not isinstance(ykey, str):
            raise PlotException('Common', 'C025', ('ykey',))
        if intkey is not None and not isinstance(intkey, str):
            raise PlotException('Common', 'C025', ('intkey',))
        if rkey is not None and not isinstance(rkey, str):
            raise PlotException('Common', 'C025', ('rkey',))

        self.keys = (xkey, ykey, intkey, rkey)

    #########################################
    def SetIntRange(self, range0, range1):
        """強度積算範囲設定処理

        self.intRangeに積算範囲を設定する。
        引数のどちらかがNoneの時は全範囲になるようにself.intRangeをNoneにする。

        Parameters:
            range0 (float or None): 積算範囲始値
            range1 (float or None): 積算範囲終値
        """
        self.intRange = format_int_range(range0, range1)

    #########################################
    def SetData(self, data=None):
        """
        Set Data
        @param data(EleentContainerArray, -Matrix or list of arrays)
        @retval None
        """
        if data is None:
            if self.data is None:
                return False
            data = self.data
        else:
            if self.data is not None:
                del self.data
            self.data = data

        if isinstance(data, Manyo.ElementContainerMatrix) or isinstance(data, Manyo.ElementContainerArray):
            self.ndata = self.converter.ConvertData(data, self.keys, self.intRange)
        elif data is not None:
            self.ndata = data
        else:
            msg = "U2IFJupyter::SetData >> ChangeData argument is invalid."
            raise UserWarning(msg)
        return True

    #########################################
    def ReadData(self, filename):
        """
        """
        print("ReadData not work yet")

    #########################################
    def SetTitles(self, title, comment=None):
        """
        Set title strings
        @param title(string)
        @retval None
        """
        self.params.title = str(title)
        if comment is not None:
            self.params.comment = comment

    #########################################
    def SetTransPosition(self, trans):
        """
        軸の転置指定
        @param  trans (bool) True means axes transpose
        @retval 無し
        """
        # 引数の型チェック
        if isinstance(trans, bool):
            if self.params.transPosition != trans:
                self.params.transPosition = trans
        else:
            msg = "U2IFJupyter::SetTransPosition >> argument is invalid."
            raise UserWarning(msg)

    #########################################
    def SetXScale(self, autoFlag, r_min=None, r_max=None):
        """
        Set Xaxis range
        @param autoFlag(bool) True means auto scale mode
        @param r_min(float) minimum value of the range (enabled when autoFlag is False) / None (autoscale)
        @param r_max(float) maximum value of the range (enabled when autoFlag is False) / None (autoscale)
        @retval None
        """
        self.params.xscale.autoRange = autoFlag
        if not autoFlag:
            if r_min is not None:
                r_min = float(r_min)
            if r_max is not None:
                r_max = float(r_max)
            self.params.xscale.range = (r_min, r_max)

    #########################################
    def SetYScale(self, autoFlag, r_min=None, r_max=None):
        """
        Set Yaxis range
        @param autoFlag(bool) True means auto scale mode
        @param r_min(float) minimum value of the range (enabled when autoFlag is False) / None (autoscale)
        @param r_max(float) maximum value of the range (enabled when autoFlag is False) / None (autoscale)
        @retval None
        """
        self.params.yscale.autoRange = autoFlag
        if not autoFlag:
            if r_min is not None:
                r_min = float(r_min)
            if r_max is not None:
                r_max = float(r_max)
            self.params.yscale.range = (r_min, r_max)

    #########################################
    def SetUnits(self, xunit=None, yunit=None):
        """
        軸の単位(unit)を指定
        @param  xunit (string or None) X軸の単位
        @param  yunit (string oe None) Y軸の単位
        @retval 無し
        """
        if xunit is not None:
            self.params.xscale.unit = xunit
        if yunit is not None:
            self.params.yscale.unit = yunit

    #########################################
    def SetLabels(self, xlabel, ylabel):
        """
        軸のラベルを指定
        @param  xlabel (string or None) X軸のラベル
        @param  ylabel (string or None) Y軸のラベル
        @retval 無し
        """
        if xlabel is not None:
            self.params.xscale.label = xlabel
        if ylabel is not None:
            self.params.yscale.label = ylabel

    #########################################
    def SetHGrid(self, grid, color=None, interval=None):
        """
        水平グリッドの指定
        @param  grid (bool) True:On / False:Off
        @param  color (int) 0:White, 1:Yellow 2:Red, 3:Black, 4:Blue, 5:Green, 6:Cyan, 7:Purple
        @param  interval (float) 間隔
        @retval 無し
        """
        # 引数の型チェック
        if isinstance(grid, bool):
            self.params.hgrid = grid
            # カラーが指定されていれば
            if color is not None:
                self.params.gridColorH = color
            # 間隔が指定されていれば
            if interval is not None:
                self.params.hgridInterval = interval

    #########################################
    def SetVGrid(self, grid, color=None, interval=None):
        """
        垂直グリッドの指定
        @param  grid (bool) True:On / False:Off
        @param  color (int) 0:White, 1:Yellow 2:Red, 3:Black, 4:Blue, 5:Green, 6:Cyan, 7:Purple
        @param  interval (float) 間隔
        @retval 無し
        """
        # 引数の型チェック
        if isinstance(grid, bool):
            self.params.vgrid = grid
            # カラーが指定されていれば
            if color is not None:
                self.params.gridColorV = color
            # 間隔が指定されていれば
            if interval is not None:
                self.params.vgridInterval = interval

    #########################################
    def SetColorMap(self, cmap):
        """
        Set color map
        @param cmap(int or string) Color map No. or name ['Default', 'autumn', 'winter', 'gray', 'haxby', 'terrain', 'turbo']
        @retval None
        """
        if isinstance(cmap, int):
            if cmap >= 0 and cmap < len(COLOR_MAPS):
                self.params.colorMap = cmap
                return
        elif isinstance(cmap, str):
            for i, a_map in enumerate(COLOR_MAPS):
                if cmap == a_map:
                    self.params.colorMap = i
                    return

        msg = "U2IFJupyter::SetColorMap >> invalid argument. Acceptable : "
        for a_name in COLOR_MAPS:
            msg += (a_name + ", ")
        msg = msg[:-2]
        raise UserWarning(msg)

    #########################################
    def SetSmooth(self, flag, times=1):
        """
        Set smoothing parameters
        @param flag(int) if flag<=0, Smoothing is off
        @param times(int) the number of times
        @retval None
        """
        if float(flag) <= 0:
            self.params.smoothing = [False, 1, 1]
        else:
            self.params.smoothing = [True, int(flag), int(times)]

    #########################################
    def SetSmoothing(self, isSmooth=False, windowVal=1, timesVal=1):
        """
        Set Smoothiing
        @param  isSmooth (bool) Smooth on or not
        @param windowVal (int) Window Size for smoothing
        @param timesVal (int) The number of times
        @retval None
        """
        if isinstance(isSmooth, bool):
            if isSmooth:
                self.params.smoothing = [True, int(windowVal), int(timesVal)]
            else:
                self.params.smoothing = [False, 1, 1]

    #########################################
    def SetBankSeparator(self, flag=False):
        """
        Bank Separatorの指定
        @param flag   Showing bank separators or not
        @retval None
        """
        # 引数の型チェック
        if isinstance(flag, bool):
            self.params.bankseparator = flag
        else:
            msg = "U2IFJupyter::SetBankSeparator >> invalid argument."
            raise UserWarning(msg)

    #########################################
    def SetIntRange(self, autoFlag, r_min=None, r_max=None):
        """
        Set Zaxis(Intensity) range
        @param autoFlag(bool) True means auto scale mode
        @param r_min(float) minimum value of the range (enabled when autoFlag is False) / None (autoscale)
        @param r_max(float) maximum value of the range (enabled when autoFlag is False) / None (autoscale)
        @retval None
        """
        self.SetRange(autoFlag, r_min, r_max)

    #########################################
    def SetRange(self, autoFlag, r_min=None, r_max=None):
        """
        Set Zaxis(Intensity) range
        @param autoFlag(bool) True means auto scale mode
        @param r_min(float) minimum value of the range (enabled when autoFlag is False) / None (autoscale)
        @param r_max(float) maximum value of the range (enabled when autoFlag is False) / None (autoscale)
        @retval None
        """
        self.params.autoRange = autoFlag
        if autoFlag:
            self.params.range = None
        else:
            if r_min is not None:
                r_min = float(r_min)
            if r_max is not None:
                r_max = float(r_max)
            self.params.range = (r_min, r_max)

    #########################################
    def SetLog(self, zlog):
        """
        Set log scale on Z axis
        @param zlog(bool) True means plot with log scale
        @retval None
        """
        if isinstance(zlog, bool):
            self.params.logScale = zlog
        else:
            msg = "U2IFJupyter::SetLog >> invalid argument."
            raise UserWarning(msg)

    #########################################
    def SetFont(self, key, fontSize=10, fontName=None):
        """
        Set font size and font
        @param key(str) target
        @param fontSize(int) font size
        @param fontName(str) font name
        @retval None
        """
        if key in self.params.fontDict.keys():
            if isinstance(fontSize, int):
                self.params.fontDict[key][0] = int(fontSize)
            else:
                msg = "U2IFJupyter::SetFont >> invalid argument."
                raise UserWarning(msg)
            if fontName is not None:
                if isinstance(fontName, str) and fontName in ["Serif", "Sans"]:
                    self.params.fontDict[key][1] = fontName
                else:
                    msg = "U2IFJupyter::SetFont >> invalid argument."
                    raise UserWarning(msg)
            else:
                pass
        else:
            msg = "U2IFJupyter::SetFont >> invalid key argument. key must be {}".format(self.params.fontDict.keys())
            raise UserWarning(msg)

    #########################################
    def SetFigureSize(self, px, py):
        """
        Set Size of Figure
        @param px(float) size of X direction
        @param py(float) size of Y direction
        @retval None
        """
        if ISJUPYTER:
            return
        self.fig = Figure((px, py), 100)
        self.canvas = FigCanvas(self.fig)

    #########################################
    def SaveFile(self, filename):
        """
        Save figure file
        @param filename(string) The path of a plot figure file.
        @retval None
        """
        if self.ndata is None:
            if not self.SetData():
                msg = "U2IFJupyter::SaveFile >> data is invalid."
                raise UserWarning(msg)

        if self.d2 is not None:
            del self.d2

        self.d2 = D2Chart(self, self.canvas, self.ndata, True)
        if os.path.isdir(os.path.dirname(filename)):
            pass
        else:
            filename = os.path.join(os.getcwd(), filename)

        self.fig.savefig(filename)
    #########################################

    def SaveAsTextFromPlot(self, filename):
        """
        データファイルをテキスト形式で保存処理
        This function is copied from u2dplot::D2Frame::SaveAsTxt.
        @param  filename (string) ファイルのパス
        @retval 無し
        """
        import codecs
        root, ext = os.path.splitext(filename)
        if ext == "":
            filename = root + ".txt"
        elif ext != ".txt":
            raise PlotException('Common', 'C043', ("txt",))
        try:
            # テキストファイルを開く
            fw = codecs.open(filename, 'w', 'utf-8')
        except:
            raise PlotException('Common', 'C018', (filename, ))
        # グラフのタイトルをコメントとして書き込む
        fw.write("# Title: " + self.params.title + os.linesep)
        # グラフのコメントを書き込む
        lines = self.params.comment.split('\n')
        if len(lines) > 0:
            fw.write("# Comment: " + lines[0] + os.linesep)
        if len(lines) > 1:
            fw.write("#          " + lines[1] + os.linesep)
        if len(lines) > 2:
            fw.write("#          " + lines[2] + os.linesep)

        # 行タイトル
        fw.write("#       X           Y         Int" + os.linesep)
        # データを展開
        if self.ndata is None:
            if not self.SetData():
                msg = "U2IFJupyter::SaveAsText >> data is invalid."
                raise UserWarning(msg)
        Z, X, Y = self.ndata
        # テキストファイルに書き込み
        for i in range(len(Z[0])):
            for j in range(len(Z)):
                xx = (X[j][i] + X[j][i + 1]) / 2.0
                yy = (Y[j][i] + Y[j + 1][i]) / 2.0
                value = Z[j][i]
                if value >= MASKVALUE:
                    value *= 1.1
                line = "%.5e %.5e %.5e" % (xx, yy, value)
                fw.write(line + os.linesep)
            fw.write(os.linesep)
        fw.close()
    #########################################

    def SaveAsText(self, filename, dQ=0.0, isIgnoreMask=False, maskinfo=[False, 0.0]):
        """
        Save data as a text file (format and arguments are completely same as M2PlotPlusCui::SaveAsText)
        @param filename (str) path/to/file
        @param dQ (float) binning of Q at elastic position which is used for inelastic powder data
        @param isIgnoreMask (bool) Whether points witch intensity is mask value are ignore ( not saved in file ) or not
        @param maskinfo (list of bool and float) [True, <val>] is to replace mask value to <val>
        @retval None
        """
        if len(maskinfo) != 2:
            raise UserWarning(
                "M2PlotPlusCui::SaveAsText >> maskinfo is invalid")
        filename = str(filename)
        saveTxt = U2IFJupyterTextFileIO(self)
        mask_val = 0.0
        try:
            mask_val = float(maskinfo[1])
        except:
            UserWarning("Invalid argument maskinfo [<bool>, <float>]")
        saveTxt.Save(filename, dQ, isIgnoreMask, [maskinfo[0], mask_val])

    #########################################

    def OnXYlimChanged(self, axes):
        """XY軸変更コールバック関数

        Parameters:
            axes (matplotlib.axes.Axes)
        """
        params = self.ifi.GetProperty('params')
        if params.transPosition:
            params.xscale.range = list(axes.get_ylim())
            params.yscale.range = list(axes.get_xlim())
        else:
            params.xscale.range = list(axes.get_xlim())
            params.yscale.range = list(axes.get_ylim())
        self.ifi.NotifyEvent(self, 'change_displayed_params')


class U2IFJupyterTextFileIO(object):
    """
    M2PlotやM2PlotPlus用のデータ保存用テキストファイルのIOクラス
    """
    ##########################################################

    def __init__(self, parent):
        # 各種初期化
        self.parent = parent
        self.XRANGE = "XRANGE"
        self.YRANGE = "YRANGE"
        self.XKEY = "XKEY"
        self.YKEY = "YKEY"
        self.XUNIT = "XUNIT"
        self.YUNIT = "YUNIT"
        self.MASKVALUE = "MASKVALUE"
        self.XLABEL = "XLABEL"
        self.YLABEL = "YLABEL"
        self.TITLE = "Title"
        self.COMMENT = "Comment"
        self.HEADER = "## M2Plot+ Output (%s)\n"
        self.DATATITLE = "## X,Y,Intensity,Error"

    ##########################################################
    def Save(self, filename, dQ=0.0, isIgnoreMask=False, maskinfo=[False, 0.0]):
        """
        テキスト保存用メソッド
        @param filename (str) 保存先のフルパス
        @param dQ (float) 非弾性散乱粉末データ用の運動量軸のbin幅
        @param isIgnoreMask (bool) 強度がMASKVALUEだった時に、その点を書き出すかどうか
        @param maskinfo (list) MASKVALUEの値をどうするかを決める [<bool>,<float>] bool : Trueのとき float の値を使用、bool : Falseのとき本来のMASKVALUEを使用
        @retval None
        """
        self.params = self.parent.ifi.GetProperty('params')
        # fd = open( filename,'w' )
        try:
            fd = open(filename, 'w')
        except:
            msg = "SaveAsText : failed to open file = %s" % (filename)
            raise UserWarning(msg)

        # プロットされたbinデータを取り出し
        xbin = self.parent.ndata[1][0]
        ybin = self.parent.ndata[2].T[0]

        # キーの取り出し : self.keys = (xkey, ykey, intkey, rkey)
        _ykey = self.parent.keys[1]
        _xkey = self.parent.keys[0]

        # クリップ範囲の初期化
        clip_region = [0.0, 0.0, 0.0, 0.0]

        # データは非弾性散乱粉末データか？
        if self.parent.converter.isInelasticPowder:
            # キーはデフォルト
            _xkey = "Q"
            _ykey = "hw"
            # スライス関数の取り出しとスライス領域のクリア
            clip = self.parent.converter.arraySlicer
            clip.ClearClipRegion()

            # dQの設定（Q範囲はデータ内最大範囲）
            clip.SetQbin(dQ, -1.0, -1.0)

            # ybin （Energy軸）で区切ってスライス設定
            for i in range(len(ybin) - 1):
                clip_region[0] = -1.0
                clip_region[1] = ybin[i]
                clip_region[2] = -1.0
                clip_region[3] = ybin[i + 1]
                clip.SetClipRegion(*clip_region)

            # スライスの実行＆結果を取り出す
            ret_eca = clip.CutAlongX(True)

        else:
            # DetectMapモードなら
            # if self.parent.isDetMapMode:
            # データがElementContainerMatrixなら
            if isinstance(self.parent.data, Manyo.ElementContainerMatrix):
                data = self.parent.converter._eca
            else:
                data = self.parent.data

            # 非弾性散乱粉末データのスライスデータの構造に合わせてArrayの構造を変換する
            ret_eca = Manyo.ElementContainerArray()
            num_of_y = len(data(0).PutXList()) - 1

            II = []
            EE = []
            for i in range(data.PutSize()):
                II.append(data(i).PutYList())
                EE.append(data(i).PutEList())

            for i in range(num_of_y):
                ii = []
                ee = []
                for j in range(len(II)):
                    ii.append(II[j][i])
                    ee.append(EE[j][i])
                ec = Manyo.ElementContainer()
                ec.Add("X", xbin.tolist())
                ec.Add("I", ii)
                ec.Add("E", ee)
                ec.SetKeys("X", "I", "E")
                ret_eca.Add(ec)

        XX = ret_eca(0).PutXList()
        YY = np.array(ybin)

        # テキストファイル書き出し開始
        t_now = datetime.datetime.now()
        fd.write(self.HEADER % (t_now.strftime('%Y-%m-%d %H:%M:%S')))
        # 領域情報
        # 非弾性粉末か？
        if self.parent.converter.isInelasticPowder:
            xrange_text = "## %s=%g,%g,%g \n" % (self.XRANGE, (XX[0] + XX[1]) / 2.0, (XX[-1] + XX[-2]) / 2.0, dQ)
        else:
            xrange_text = "## %s=%g,%g,%g \n" % (self.XRANGE, (XX[0] + XX[1]) / 2.0, (XX[-1] + XX[-2]) / 2.0, XX[1] - XX[0])

        yrange_text = "## %s=%g,%g,%g \n" % (self.YRANGE, (YY[0] + YY[1]) / 2.0, (YY[-1] + YY[-2]) / 2.0, (ybin[1] - ybin[0]))
        fd.write(xrange_text)
        fd.write(yrange_text)

        # ラベル情報
        fd.write("## %s=%s\n" % (self.XLABEL, self.params.xscale.label))
        fd.write("## %s=%s\n" % (self.YLABEL, self.params.yscale.label))

        # UNIT情報
        fd.write("## %s=%s\n" % (self.XUNIT, self.params.xscale.unit))
        fd.write("## %s=%s\n" % (self.YUNIT, self.params.yscale.unit))

        # マスク情報
        if not isIgnoreMask:
            if maskinfo[0]:
                fd.write("## %s=%g\n" % (self.MASKVALUE, maskinfo[1]))

        # 軸のキー名
        fd.write("## %s=%s\n" % (self.XKEY, _xkey))
        fd.write("## %s=%s\n" % (self.YKEY, _ykey))

        # タイトル・コメント
        p_title = self.params.title
        p_comm = self.params.comment
        fd.write("# %s=%s\n" % (self.TITLE, p_title))
        fd.write("# %s=%s\n" % (self.COMMENT, p_comm))

        # データ
        fd.write(self.DATATITLE + " \n")
        for i in range(ret_eca.PutSize()):
            yy = (ybin[i] + ybin[i + 1]) / 2.0
            if yy < 1e-13 and yy > -1e-13:
                yy = 0.0
            ec = ret_eca(i)
            XX = ec.PutXList()
            YY = ec.PutYList()
            EE = ec.PutEList()
            for j in range(len(YY)):
                xx = (XX[j] + XX[j + 1]) / 2.0
                ii = YY[j]
                ee = EE[j]
                if ii < mm.MLF_MASKVALUE:
                    fd.write("%g, %g, %g, %g\n" % (xx, yy, ii, ee))
                else:
                    if isIgnoreMask:
                        continue
                    if maskinfo[0]:
                        fd.write("%g, %g, %g, %g\n" %
                                 (xx, yy, maskinfo[1], 0.0))
                    else:
                        fd.write("%g, %g, %g, %g\n" % (xx, yy, ii, ee))
        # ファイル閉
        fd.close()

if __name__ == '__main__':
    import Cmm
    ecm = Cmm.LoadDataFromDump("./", "Load373.dmp")
    p = U2IFJupyter(ecm, 1)

    p.SetKeys(None, None, None)
    p.SetFigureSize(5.0, 5.0)
    p.SetRange(False, 0, 40.0)
    p.SetTitles("U2IFJupyter Test", "YAHOOO!")
    p.SetLabels("|Q| (1/Ang.)", "Energy (meV)")
    p.SetColorMap("haxby")
    p.SaveFile(os.path.join(os.getcwd(), "U2IFJupyter.png"))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
import Manyo.Utsusemi as mu
import Manyo  # [inamura 160804]
import os  # [inamura 160804]
import math  # [inamura 120806]


class TreatSPE(object):
    """
    ISISやMSLICEで用いられるSPE fileおよびPHX fileの作成、読み込みを行うクラス
    ElementContainerMatrixからSPEおよびPHXファイルを作成する
    """

    def __init__(self):
        """
        コンストラクタ
        @param Nothing
        """
        self.CommentHead = "TreatSPE>>> "

    def _MakeOneGroup(self, in_list):
        """
        SPEファイル内で使用される8列のフォーマットの作成し、ストリングで戻す
        @param   in_list (LIST) データ
        @retval  (String) フォーマット化されたデータ
        """
        num = len(in_list)
        rows = int(num / 8)
        amari = num % 8
        # print "lines,colms,amari=",num,colms,amari

        SS = ""
        for i in range(rows):
            # SS += "%-10.4g%-10.4g%-10.4g%-10.4g%-10.4g%-10.4g%-10.4g%-10.4g\n" \
            # SS += "%-10.4f%-10.4f%-10.4f%-10.4f%-10.4f%-10.4f%-10.4f%-10.4f\n" \
            SS += "%10.3E%10.3E%10.3E%10.3E%10.3E%10.3E%10.3E%10.3E\n" % (in_list[i * 8],
                                                                          in_list[i * 8 + 1],
                                                                          in_list[i * 8 + 2],
                                                                          in_list[i * 8 + 3],
                                                                          in_list[i * 8 + 4],
                                                                          in_list[i * 8 + 5],
                                                                          in_list[i * 8 + 6],
                                                                          in_list[i * 8 + 7])
        if amari != 0:
            for i in range(amari):
                SS += "%10.3E" % in_list[8 * rows + i]
            SS += "\n"

        return SS

    def MakeSPE(self, dat, filename="", Def_Pixel_Size=[], phx_filepath="default", par_filepath="default"):
        """
        SPE,PHXファイルを作成する
        ElementContainerMatrixから必要な情報を取り出し、必要なフォーマットへ変換し、ファイルに保存する
        @param   dat   (ElementContainerMatrix) TofToEnergyTransferが終わっている必要がある
        @param   file  (string) ファイル名（拡張子なし）
        @retval  None
        """
        numQ = 0
        Int_list = []  # List of the intensity list put from all ElementContainers
        Err_list = []  # List of the error list put from all ElementContainers
        Pol_list = []  # List of the polar angles put from all ElementContainers
        Azm_list = []  # List of the azimuthal angles put from all ElementContainers
        L2_list = []  # List of L2 #[inamura 120806]

        # エレメントコンテナを取り出す
        print(self.CommentHead + "Loading ElementContainerMatix...")
        for psd in range(dat.PutTableSize()):
            eca = dat(psd)
            # PSDはMASKEDか
            hh_eca = eca.PutHeaderPointer()
            if (hh_eca.CheckKey("MASKED") == 1) and (hh_eca.PutInt4("MASKED") == 1):
                continue
            # MASKEDでなければPixelのECを取り出す
            for pixel in range(eca.PutTableSize()):
                # PixelはMASKEDか
                if (hh_eca.CheckKey("MASKED") == 1) and (hh_eca.PutInt4("MASKED") == 1):
                    continue
                # エレメントコンテナの取り出しと単位量あたりへの変換
                ec_org = eca(pixel)
                hh_ec = ec_org.PutHeaderPointer()
                # Polar angleとAzimuthal angleの取り出し
                if (hh_ec.CheckKey("MASKED") == 1) and (hh_ec.PutInt4("MASKED") == 1):
                    continue
                pol_vec = hh_ec.PutDoubleVector("PixelPolarAngle")
                azm_vec = hh_ec.PutDoubleVector("PixelAzimAngle")
                pol = [pol_vec[0], pol_vec[1]]
                azm = [azm_vec[0], azm_vec[1]]
                Pol_list.append(pol)
                Azm_list.append(azm)

                # [inamura 120806]-->
                # Pixel Position
                pos_vec = hh_ec.PutDoubleVector("PixelPosition")
                L2_list.append(math.sqrt(pos_vec[0] * pos_vec[0] + pos_vec[1] * pos_vec[1] + pos_vec[2] * pos_vec[2]) / 1000.0)
                # <--[inamura 120806]

                # IntensityとErrorの取り出し
                ec = mu.HistogramBinToPoint(ec_org)
                Int_list.append(ec.PutYList())
                Err_list.append(ec.PutEList())
                numQ += 1

        # Energy Transferはどのヒストグラムでも同一だとして取り出す
        hw_list = dat(0, 0).PutXList()
        # Phiの値を計算（gridなので個数は+1）
        phi_list = [(Pol_list[0][0] - (Pol_list[1][0] - Pol_list[0][0]) / 2.0)]
        for i in range(len(Pol_list) - 1):
            phi_list.append((Pol_list[i][0] + Pol_list[i + 1][0]) / 2.0)
        phi_list.append(Pol_list[-1][0] + (Pol_list[-1][0] - Pol_list[-2][0]) / 2.0)

        # print "numQ, length of hw, phi, Int, Err, pols, azms = ",numQ,len(hw_list),len(phi_list),len(Int_list),len(Err_list),len(Pol_list),len(Azm_list)

        # ファイルを開く
        # SPE_filename = filename+".spe"
        # PHX_filename = filename+".phx"
        # PAR_filename = filename+".par" #[inamura 120806]
        ROOT_filename = ""
        if filename[-4:].upper() == ".SPE":
            SPE_filename = filename
            ROOT_filename = filename[:-4]
        else:
            SPE_filename = filename + ".spe"
            ROOT_filename = filename
        if phx_filepath.upper() == "NONE":
            PHX_filename = ""
        elif phx_filepath.upper() == "DEFAULT":
            PHX_filename = ROOT_filename + ".phx"
        else:
            if phx_filepath[-4:].upper() == ".PHX":
                PHX_filename = phx_filepath
            else:
                PHX_filename = phx_filepath + ".phx"
        if par_filepath.upper() == "NONE":
            PAR_filename = ""
        elif par_filepath.upper() == "DEFAULT":
            PAR_filename = ROOT_filename + ".par"
        else:
            if par_filepath[-4:].upper() == ".PAR":
                PAR_filename = par_filepath
            else:
                PAR_filename = par_filepath + ".par"

        # fo_phx = open(PHX_filename,'w')
        # fo_spe = open(SPE_filename,'w')
        # fo_par = open(PAR_filename,'w') #[inamura 120806]

        # PHXファイルの作成
        if PHX_filename != "":
            fo_phx = open(PHX_filename, 'w')
            print(self.CommentHead + "Making PHX file... %s" % (PHX_filename))
            fo_phx.write(("%d\n" % len(Pol_list)))
            i = 1
            SS = ""
            for l2, pol, azm in zip(L2_list, Pol_list, Azm_list):
                # SS += "%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8d\n" % (10.0,0.0,pol[0],azm[0],pol[1],azm[1],i)
                # SS += " %9.4f %9.4f %9.4f %9.4f %9.4f %9.4f %9d\n" % (10.0,0.0,pol[0],azm[0],pol[1],azm[1],i)
                # SS += " %9.4f %9.4f %9.4f %9.4f %9.4f %9.4f %9d\n" % (10.0,0.0,pol[0],azm[0],pol[1]*2.0,azm[1]*2.0,i)  ##[inamura 120511]
                SS += " %9.4f %9.4f %9.4f %9.4f %9.4f %9.4f %9d\n" % (l2, 0.0, pol[0], azm[0], (pol[1] * 2.0), (azm[1] * 2.0), i)
                i += 1
            fo_phx.write(SS)
            fo_phx.close()

        # [inamura 120806]-->
        # PAR file
        if PAR_filename != "":
            fo_par = open(PAR_filename, 'w')
            if len(Def_Pixel_Size) < 2:
                hh = dat.PutHeaderPointer()
                if hh.CheckKey("TypicalPixelSize") == 0:
                    msg = "TreatSPE : There is no pixel size in given data"
                    raise UserWarning(msg)
                ps = hh.PutDoubleVector("TypicalPixelSize")
                Def_Pixel_Size = [ps[0], ps[1], ps[2]]

            print("len(Def_Pixel_Size)={}".format(len(Def_Pixel_Size)))
            if len(Def_Pixel_Size) >= 2:
                det_width = Def_Pixel_Size[1] / 1000.0  # [mm]->[m]
                det_height = Def_Pixel_Size[0] / 1000.0  # [mm]->[m]
                print(self.CommentHead + "Making PAR file... {}".format(PAR_filename))
                fo_par.write("%d\n" % len(L2_list))
                i = 1
                SS = ""
                for j in range(len(L2_list)):
                    # tmp = ( L2_list[j], Pol_list[j][0], Azm_list[j][0], det_width, det_height, i )
                    tmp = (L2_list[j], Pol_list[j][0], -1.0 * Azm_list[j][0], det_width, det_height, i)
                    SS += "%9.4f\t%9.4f\t%9.4f\t%9.4f\t%9.4f\t%9d\n" % tmp
                    i += 1
                fo_par.write(SS)
            fo_par.close()
        # <--[inamura 120806]

        # SPEファイルの作成
        print(self.CommentHead + "Making SPE file... %s" % (SPE_filename))
        fo_spe = open(SPE_filename, 'w')
        fo_spe.write(("%d %d\n" % (numQ, (len(hw_list) - 1))))

        fo_spe.write("### Phi Grid\n")

        SS = self._MakeOneGroup(phi_list)
        fo_spe.write(SS)

        fo_spe.write("### Energy Grid\n")
        SS = self._MakeOneGroup(hw_list)
        fo_spe.write(SS)

        counts = 0
        total_num = len(Int_list)
        for intensity, error in zip(Int_list, Err_list):
            # 進捗状況を示す
            if ((counts % 1000) == 0):
                print(self.CommentHead + "  %10d/%d" % (counts, total_num))
            SS = self._MakeOneGroup(intensity)
            fo_spe.write("### S(Phi,w)\n")
            fo_spe.write(SS)

            SS = self._MakeOneGroup(error)
            fo_spe.write("### Errors\n")
            fo_spe.write(SS)
            counts += 1

        print(self.CommentHead + "  %10d/%d" % (counts, total_num))
        fo_spe.close()

        print(self.CommentHead + ".....completed.")

    # [inamura 160804]-->
    def _LoadOneBlock(self, SS):
        """
        SPEファイル内で使用される8列のテキストフォーマットをデコードする
        @param   SS (String LIST) 文字列データ
        @retval  (LIST) 数値データ
        """
        ret_list = []
        for a_line in SS:
            ind = 0
            while(True):
                if len(a_line) < (10 * ind + 10):
                    break
                ret_list.append(float(a_line[(10 * ind):(10 * ind + 10)]))
                ind += 1

        return ret_list

    def _LoadOneBlockForPhx(self, SS):
        """
        PHXファイル内で使用されるテキストフォーマットをデコードする
        @param   SS (String LIST) 文字列データ
        @retval  (LIST) 数値データ
        """
        ret_list = []
        for a_line in SS:
            ind = 0
            a_line = a_line.replace("\t", " ")
            list_line = a_line.split(" ")
            for item in list_line:
                if item != "":
                    ret_list.append(float(item))

        return ret_list

    def LoadSPE(self, spepath, Ei=100.0, num_of_pixels=100, runNo="XXX000001", phx_fullpath="default", par_fullpath="default"):
        """
        SPEファイルを読み込んでElementContainerMatrixを作成する
        SPEファイルだけを指定する
        PHXファイルやPARファイルは拡張子だけが異なるファイル名と仮定している
        @param spefile (string) full path to SPE file
        @param Ei (float) Incident Energy
        @param num_of_pixels (int)  number of pixels in one detector
        @param runNo (string) Run Number (including instrument code)
        @param phx_fullpath (string) full path to PHX file
        @param par_fullpath (string) full path to PAR file
        @retval (ElementContainerMatrix)
        """
        if not os.path.exists(spepath):
            msg = "SPE file not found ({})".format(spepath)
            raise UserWarning(msg)
            return

        spefile = spepath
        rootpath = spepath[:-4]
        phxfile = ""
        parfile = ""
        if phx_fullpath.upper() == "DEFAULT":
            if os.path.exists(rootpath + ".phx"):
                phxfile = rootpath + ".phx"
            elif os.path.exists(rootpath + ".PHX"):
                phxfile = rootpath + ".PHX"
            else:
                msg = "PHX file not found ({}.phx)".format(rootpath)
                raise UserWarning(msg)
                return
        else:
            if os.path.exists(phx_fullpath):
                phxfile = phx_fullpath
            else:
                msg = "PHX file not found ({})".format(phx_fullpath)
                raise UserWarning(msg)
                return

        if par_fullpath.upper() == "DEFAULT":
            if os.path.exists(rootpath + ".par"):
                parfile = rootpath + ".par"
            elif os.path.exists(rootpath + ".PAR"):
                parfile = rootpath + ".PAR"
            else:
                msg = "PAR file not found ({}.par)".format(rootpath)
                raise UserWarning(msg)
                return
        else:
            if os.path.exists(par_fullpath):
                parfile = par_fullpath
            else:
                msg = "PAR file not found ({})".format(par_fullpath)
                raise UserWarning(msg)
                return

        print("Load SPE file : %s" % (spefile))
        print("Load PHX file : %s" % (phxfile))
        f_spe = open(spefile, "r")
        f_phx = open(phxfile, "r")
        f_par = None
        if (parfile != ""):
            print("Load PAR file : %s" % (parfile))
            f_par = open(parfile, "r")

        L2_list = []
        Pol_list = []
        Azm_list = []
        DW_list = []
        DH_list = []

        try:
            runNo_num = int(runNo)
            runNo_code = "XXX"
        except ValueError:
            runNo_code = runNo[:3]
            runNo_num = int(runNo[3:])

        # read PHX
        numQ_phx = 0
        fst_line = f_phx.readline()
        numQ_phx = int(fst_line.strip())

        for i in range(numQ_phx):
            a_line = f_phx.readline()
            a_line_list = self._LoadOneBlockForPhx([a_line])
            Pol_list.append([a_line_list[2], a_line_list[4] / 2.0])
            Azm_list.append([a_line_list[3], a_line_list[5] / 2.0])
        f_phx.close()

        # read PAR
        if f_par is not None:
            numL2_par = 0
            fst_line = f_par.readline()
            numL2_par = int(fst_line.strip())
            if numL2_par != numQ_phx:
                msg = "PHX and PAR files are inconsistency."
                raise UserWarning(msg)
                return

            isOldForm = True
            for i in range(numL2_par):
                a_line = f_par.readline()
                if isOldForm:
                    try:
                        a_line_list = self._LoadOneBlock([a_line])
                    except:
                        mu.UtsusemiMessage("PAR file is newer format.")
                        isOldForm = False
                        a_line_list = self._LoadOneBlockForPhx([a_line])
                else:
                    a_line_list = self._LoadOneBlockForPhx([a_line])
                L2_list.append(a_line_list[0])
                Pol_list[i] = [a_line_list[1], a_line_list[3]]
                # Azm_list[i] = [a_line_list[2],a_line_list[4]]
                Azm_list[i] = [-1.0 * a_line_list[2], a_line_list[4]]
                DW_list.append(a_line_list[3])
                DH_list.append(a_line_list[4])

        # read SPE
        first_line = f_spe.readline()
        first_line_list = []
        for item in first_line.strip().split(" "):
            if item != "":
                first_line_list.append(item)

        if len(first_line_list) != 2:
            raise UserWarning("This file is not SPE file.({})".format(spefile))

        numQ = int(first_line_list[0])
        numHW_bin = int(first_line_list[1]) + 1

        if numQ != numQ_phx:
            msg = "PHX and SPE files are inconsistency."
            raise UserWarning(msg)
            return

        numOfRows_Q = int(numQ / 8)
        numOfRows_HW = int((numHW_bin - 1) / 8)
        if (numQ % 8) != 0:
            numOfRows_Q += 1
        if ((numHW_bin - 1) % 8) != 0:
            numOfRows_HW += 1

        pixelId = 0
        PHI_list = []
        HW_list = []
        int_list = []
        err_list = []

        hh = Manyo.HeaderBase()
        hh.Add("RUNNUMBER", str(runNo_num))
        hh.Add("INSTRUMENT", runNo_code)
        hh.Add("Ei", Ei)
        hh.Add("isHistogram", 0)  # [inamura 210118]
        ecm = Manyo.ElementContainerMatrix(hh)
        detId = 0
        hh = Manyo.HeaderBase()
        hh.Add("MASKED", 0)
        hh.Add("DETID", detId)
        eca = Manyo.ElementContainerArray(hh)
        while(True):
            a_line = f_spe.readline()
            if a_line == "":
                ecm.Add(eca)
                break

            if a_line.find("### Phi") == 0:
                SS = []
                for i in range(numOfRows_Q):
                    SS.append(f_spe.readline())
                PHI_list = self._LoadOneBlock(SS)

            if a_line.find("### Ene") == 0:
                SS = []
                for i in range(numOfRows_HW):
                    SS.append(f_spe.readline())
                HW_list = self._LoadOneBlock(SS)

            if a_line.find("### S(P") == 0:
                SS = []
                for i in range(numOfRows_HW):
                    SS.append(f_spe.readline())
                int_list = self._LoadOneBlock(SS)

            if a_line.find("### Err") == 0:
                SS = []
                for i in range(numOfRows_HW):
                    SS.append(f_spe.readline())
                err_list = self._LoadOneBlock(SS)

            if len(int_list) != 0 and (len(int_list) == len(err_list)):
                ec = Manyo.ElementContainer()
                hh = ec.PutHeaderPointer()
                hh.Add("PIXELID", pixelId)
                pa = Manyo.MakeDoubleVector(2)
                aa = Manyo.MakeDoubleVector(2)
                pa[0] = Pol_list[pixelId][0]
                pa[1] = Pol_list[pixelId][1]
                aa[0] = Azm_list[pixelId][0]
                aa[1] = Azm_list[pixelId][1]
                hh.Add("PixelPolarAngle", pa)
                hh.Add("PixelAzimAngle", aa)
                hh.Add("DETID", detId)
                hh.Add("MASKED", 0)

                # [inamura 210118] removed
                # deltaHW = HW_list[1]-HW_list[0]
                # for i in range(len(int_list)):
                #    int_list[i] = int_list[i]*deltaHW

                ec.Add("Energy", HW_list, "meV")
                ec.Add("Intensity", int_list, "counts")
                ec.Add("Error", err_list, "counts")
                ec.SetKeys("Energy", "Intensity", "Error")

                int_list = []
                err_list = []

                eca.Add(ec)
                pixelId += 1

            if pixelId != 0 and pixelId % num_of_pixels == 0 and eca.PutSize() != 0:
                ecm.Add(eca)
                detId += 1

                hh = Manyo.HeaderBase()
                hh.Add("MASKED", 0)
                hh.Add("DETID", detId)
                eca = Manyo.ElementContainerArray(hh)

        return ecm
    # <--[inamura 160804]



#! /usr/bin/env python
# -*- coding:utf-8 -*-
import Tkinter
import datetime
import time
import os
import shutil
import Manyo as mm
import Cmm
import vis.MPlot as MP
import vis.M2Plot as MP2
import ElasticScanGUIBase as ESB
import T0fileMake as T0M
import subprocess
import ElasticScan 

CommentHeader = "ESCUI -->>  "

def StartElasticScan(RunNo, TimeParam):
    Ebin = 0.002
    Emin = -0.004
    Emax = 0.004
    Qmin = 0.0
    Qmax = 2.0
    Qbin = 0.05 
    LambdaCenter = 6.321
    dLambda = 4.15
    Tch = "D5"
    inifile = "ESCUIParams.ini"

    HWParam ="%.4f,%4f,%.6f" %(Ebin,Emin,Emax)
    HWParam2 ="%.4f,%4f" %(Emin,Emax)
    QParam = "%.2f,%.2f,%s" %(Qmin,Qmax,Qbin)
    LambdaParam = "%.4f,%.4f" %(LambdaCenter,dLambda)
    print CommentHeader, "HWParam: %s" %(HWParam)
    print CommentHeader, "HWParam2: %s" %(HWParam2)
    print CommentHeader, "LambdaParam: %s"  %(LambdaParam)
    print CommentHeader, "Qparam: %s" %(QParam)
   
   
    #TimeParam = "%s,%s,%s" %(self.StartTime,self.EndTime,self.TimeBin)
    #TempParam = "%s,%s,%s" %(self.StartTemp,self.EndTemp,self.TempBin)
    print CommentHeader, "TimeParam: %s" %(TimeParam)
    print CommentHeader, "TempCh: %s" %(Tch)
    #print self.CommentHeader, "TempParam: %s" %(TempParam)

    #dataPath = "/home/dnaadmin/DNA/tmp/" #T.Yamada[130218]
    ECMbstName = "ECM_%d_ES.bst" %(RunNo)
    
    FileHeader = "ESGUIRun"
    ESName = "%s%dES.txt" % (FileHeader, RunNo)
    ESTimeName = "%s%dEStime.txt" % (FileHeader, RunNo)
    TIName = "%s%dTI.txt" % (FileHeader, RunNo)
    TITimeName = "%s%dTItime.txt" % (FileHeader, RunNo)
    TempLogName = "%s%dTempLog.txt" %(FileHeader, RunNo)

    copyresult, messg =CopyEventData(int(RunNo))
    print messg 

    print type(RunNo), RunNo
    T0M.MakeT0file(str("%06d" %int(RunNo)))
#    return -1
    print CommentHeader, "ECM make start"
    ECM = ElasticScan.GetQETimeECMMulth(RunNo, HWParam, LambdaParam, QParam, TimeParam, inifile)
    print CommentHeader, "ECM make end"
    ECM = ElasticScan.ReadECM(ECMbstName)
    EC = ElasticScan.ElasticScanTotalIntensity(ECM, HWParam2, QParam, inifile)
    print CommentHeader, "Total Intensity vs Time"
    ElasticScan.OutputECText(EC, TITimeName)
    ElasticScan.OutPutElasticScanECM(ECM)
    del ECM
#    showplot(EC)
    EC = Cmm.CheckSampleTempByRunNumber(RunNo,Tch)
    Cmm.OutputECText(EC,"_"+Tch)
    print CommentHeader, "Finished"
    return -1
#    #----------------- 
#    #u2 vs time is calculated
#    ECA = ElasticScan.ElasticScanQ2LnI(ECM, HWParam2, QParam, inifile)
#    EC1 = ElasticScan.ElasticScanFit(ECA, "p1")
#    ElasticScan.OutputECText(EC1, ESTimeName)
#    self.showplot(EC1)
#    del ECM
#    del ECA
#    #Time is converted to temperature.
#    EC2 = ElasticScan.ElasticScanCheckTempNew(EC, self.Tch)
#    EC4 = ElasticScan.ElasticScanXAxisChangeToTemp(EC, EC2, TempParam)
#    ElasticScan.OutputECText(EC2,TempLogName) #Temperature log
#    ElasticScan.OutputECText(EC4, TIName) #TotalIntensity vs Temperature
#    EC3 = ElasticScan.ElasticScanXAxisChangeToTemp(EC1, EC2, TempParam)
#    ElasticScan.OutputECText(EC3, ESName) #u2 vs Temperature
#    print self.CommentHeader, "ElasticScan Reduction End"
#    print self.CommentHeader, "I(Q) vs time is saved as \n%sRun%sIQ.txt\n" %(dataPath,self.RunNo) 
#    print self.CommentHeader, "u2 is saved as \n%s%s\n%s%s\n" %(dataPath, ESName, dataPath, ESTimeName)
#    print self.CommentHeader, "Total intensity is saved as \n%s%s\n%s%s\n" %(dataPath, TIName, dataPath, TITimeName)
#    print self.CommentHeader, "Temperature Log is saved as \n%s%s\n" %(dataPath, TempLogName)
#    
#    self.showplot(EC3)

def showplot(dat):
    if type(dat) == type(mm.ElementContainer()):
        MP.MPlot(dat)
    elif type(dat) == type(mm.ElementCotainerArray()):
        MP2.M2Plot(dat)
    else:
        print CommentHeader, "dat is not ElementContainer or ElementContainerArray"
        print CommentHeader, "dat type is ", type(dat)

def CopyEventData(RunNo):
    command = "ssh dnaadmin@192.168.1.10 /home/dnaadmin/Yamada/01_CopyEventData/CopyEventEdata.sh %d" %(RunNo)
    args = command.split(" ")
    proc = subprocess.Popen(args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
    stdout_data, stderr_data = proc.communicate()
    print "stdout_data >>", stdout_data
    print "stderr_data >>", stderr_data.replace("\n","")
    result = proc.returncode
    #proc.close()
    if result == 1:
        messg =  "DNA%06d_* is not found on /data and /ifs_data/DNA/DNA" %RunNo
    else:
        messg =  "DNA%06d_* is ready for the reduction." %RunNo
    return result, messg


if __name__ == "__main__":
    FastChopperUse = True
    print "ElasticScan Reduction ->> Input RunNo"
    RunNo = raw_input()
    print "ElasticScan Reduction ->> Input TimeParam, StartTime,EndTime,TimeBin"
    TimeBin = raw_input()
    StartElasticScan(int(RunNo), TimeBin)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

import time
import os
import glob
import Manyo
import Manyo.Utsusemi as mu
import Manyo.SAS as ms
import math

import utsusemi.ana.Reduction.BaseCommands as BaseCom
import Hist, Corr, QSpace
import utsusemi.SAS.ana.Reduction.QSpaceBaseCommands as QBC
import utsusemi.SAS.ana.Reduction.HistBaseCommands as HBC

# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
ecs = None
EC = None
ret = None
PyObj = None
Iq = None
Tr = None
ECA = None
DATList=None

#########################################
def GetNeunetHistSAS(runNo="0",HistParam="tof,0,40000,100",DetParam="psd",DetRange="All",TimeRange="All",FrameInfo="None",MaskInfo="NoFile",CaseInfo="None",BGInfo="TimeDep:None",TofShift="None",AnaMode="0",ParamFiles="-,-",SolAngCor=False,Options="-"):
    """
    Convert Event data of NEUNET to Histogram
    GetNeunetHist(runNo="197,198",
                  HistParam="tof,0,40000,100",
                  DetParam="psd",
                  DetRange="All",
                  TimeRange="All",
                  FrameInfo="None",
                  MaskInfo="NoFile"
                  CaseInfo="None",
                  BGInfo="TimeDep:None",
                  TofShift="None",
                  AnaMode="0",
                  ParamFiles="-,-",
                  SolAngCor=False,
                  Options="-"
                  )

    -----------------------
    HistParam format
        TOF
           delta-TOF const :  tof,<min_tof>,<max_tof>,<delta_tof>
           d-TOF/TOF const : rtof,<min_tof>,<max_tof>,<racio>

        TOF with time forcusing
           delta-TOF const : tf-tof,<min_tof>,<max_tof>,<delta_tof>
           d-TOF/TOF const : tf-rtof,<min_tof>,<max_tof>,<racio>

        Lambda
           delta-lambda const        :  lambda,<min_lambda>,<max_lambda>,<delta_lambda>
           delta-lambda/lambda const : rlambda,<min_lambda>,<max_lambda>,<racio>

        Energy            : energy,<min_energy>,<max_energy>,<delta_energy>

        Momentum Transfer : q,<min_Q>,<max_Q>,<delta_Q>[,<ki_x>,<ki_y>,<ki_z>]

        Energy Transfer   : hw,<Ei>,<min_hw>,<max_hw>,<delta_hw>[,<t0_shift>]

        d-value
            delta-d-value                 : d,<min_d-val>,<max_d-val>,<delta_d-val>
            delta-d-value / d-value const : rd,<min_d-val>,<max_d-val>,<ratio>

    -----------------------
    DetParam format
        "<detType>[[<LLD_min>:<LLD_max>]][,..]" : detType in {"PSD","MON","PSD-PH","MON-PH", "UPSD", "TRIGNET"}
                                                "-PH" means that Pulse Height is calculated at the same time
    -----------------------
    DetRange format
        "All[:<number_of_pixels>]"                     : use all detectors
        "<startDetId>-<endDetId>[:<number_of_pixels>]" : use detectors from <startDetId> to <endDetId>
                                                         "X-Y:Z,A-B:C,D-E:F..." can be used for more settings
    -----------------------
    TofShift format
        "<ptnId>[,<p1>,<p2>,...]"
             ptnId = 0 : no use in this facade( only use if scattered data of tof origin shift vs lambda is given )
             ptnId = 1 : use shift = p1/(exp(-p2*(lambda-p3)) +p4) + p5*lambda
                         default p1=5.0, p2=8.464, p3=2.08765, p4=1.23477, p5=5.32657
    -----------------------

    @param runNo     (string or int) Run Number(s) 123 or "123,124"
    @param HistParam (string) "type,param,param,param,..." type must be in "tof" "rtof" "lambda" "rlambda" "hw" "energy" "q" "d" "rd"
    @param DetParam  (string) "DetType[,LLD,HLD]" DetType must be in "psd","n2mon","psd-ph" or "n2mon-ph"
    @param DetRange  (string) "All[:number_of_pixels]" or "startPsdId-endPsdId[:number_of_pixels]"
    @param TimeRange (string) "All" or "startTime,endTime" in [second] or "YYYY/MM/DD,hh:mm:ss,YYYY/MM/DD,hh:mm:ss"
    @param FrameInfo (string) "None" or "FrameNumber, boudaryTime"
    @param MaskInfo  (string) "<maskfile> [,<TOFmask>-<TOFmask>,..]" : if no use maskfile, set "NoFile"
    @param CaseInfo  (string) CaseInfo file must be in ${UTSUSEMI_USR_DIR}/ana/xml or ${UTSUSEMI_HOME_DIR}/ana/xml
    @param BGInfo    (string) "TimeDep:None" TimeDep:tof1-tof2
    @param TofShift  (string) "None" or "<ptnId> [,<p1>,<p2>,...]"
    @param AnaMode   (string or int) change analysis mode, which means to swich WiringInfo and DetectorInfo to be used for histogram.
    @param ParamFiles(string) "<path/to/WiringInfo>,<path/to/DetectorInfo>"
    @param SolAngCor (bool) True for doing SolidAngle Correction
    @retval DAT (ElementContainerMatrix)
    """
    runNo_strlist=runNo.split(",")
    runNoList = []
    for a_run_st in runNo_strlist:
        runNoList.append( int(a_run_st) )
    num_of_runs = len(runNoList)
    all_data_list = []
    if DetRange.find("RPMT") == 0:
        if Options.upper().find("ISCHECKPULSEID:FALSE") < 0:
            if Options == "-":
                Options = "ISCHECKPULSEID:false"
            else:
                Options += ",ISCHECKPULSEID:false"
        print("##########################[inamura 260106] Check PulseID = Flase ({})".format(Options))
        RPMT_LLDHLD = [0, 0, 0, 0] #[<LLD_X>,<HLD_X>,<LLD_Y>,<HLD_Y>]
        RPMT_COI = 32 #<LmtCoi>

        #detType="PSD"
        mode_no=5
        AnaMode = "5"
        rpmt_params = DetRange.split(",")
        if len(rpmt_params)>1:
            RPMT_LLDHLD[0] = int(rpmt_params[1])
        if len(rpmt_params)>2:
            RPMT_LLDHLD[1] = int(rpmt_params[2])
        if len(rpmt_params)>3:
            RPMT_LLDHLD[2] = int(rpmt_params[3])
        if len(rpmt_params)>4:
            RPMT_LLDHLD[3] = int(rpmt_params[4])
        if len(rpmt_params)>5:
            RPMT_LLDHLD[4] = int(rpmt_params[5])
        if len(rpmt_params)>6:
            RPMT_COI = int(rpmt_params[6])
        for a_runNo in runNoList:
            DaqId = 13 #Default
            ind = DetRange.find(":")
            if ind > 0:
                DaqId = int(DetRange[ind+1:])
            #ConvertFromRPMTK2Psd( int(a_runNo), DaqId, 0, "SAS03")
            dfile_found_path = ""
            HBC.ConvertFromRPMTK2Psd(int(a_runNo), DaqId, 0, "SAS03", mode_no, RPMT_LLDHLD, RPMT_COI, dfile_found_path)

    AnaMode = AnaMode+","+ParamFiles
    det_range_list = DetRange.split(",")
    DetRange = ""
    for a_det in det_range_list:
        a_det = a_det.strip()
        if a_det == "SM":
            DetRange += "SMBL14,SMBL16,"
        else:
            DetRange += (a_det + ",")
    DetRange = DetRange[:-1]
    print("#[inamura 240411] DetRange rewrite to ", DetRange)
    for runNo in runNoList:
        HBC.CheckT0bUpdate(int(runNo))
        a_dat_list = BaseCom.GetNeunetHist(runNo,HistParam,DetParam,DetRange,TimeRange,FrameInfo,MaskInfo,CaseInfo,BGInfo,TofShift,AnaMode,Options)
        if type(a_dat_list)!=list:
            a_dat_list=[a_dat_list]
        all_data_list.append(a_dat_list)
    dat_list = []
    UCC=mu.UtsusemiCalcContainers()

    for i in range(len(all_data_list[0])):
        ret = Manyo.ElementContainerMatrix(all_data_list[0][i])
        kickers=ret.PutHeaderPointer().PutInt4("KICKERCOUNT")
        for j in range(num_of_runs-1):
            a_dat = all_data_list[j+1][i]
            kickers += a_dat.PutHeaderPointer().PutInt4("KICKERCOUNT")
            ret_tmp = Manyo.ElementContainerMatrix(ret.PutHeader())
            if UCC.CalcContainers(ret_tmp,1.0,ret,"+",1.0,a_dat):
                pass
            else:
                raise
            ret = ret_tmp
        ret.InputHeader(all_data_list[0][i].PutHeader())
        ret.PutHeaderPointer().OverWrite("KICKERCOUNT",kickers)
        dat_list.append(ret)
    del UCC

    for dat in dat_list:
        ## Create Lambda Vector for each EC
        tt1 = ms.CreateLambVector( dat )
        tt1.Execute()
        del tt1

        ## Convertion from histogram to per_unit
        tt2 = ms.ConvertToPoint(dat)
        tt2.Execute()
        del tt2

        ## Solid angle correction
        if (SolAngCor):
            for i in range(dat.PutSize()):
                if (dat(i).PutHeaderPointer().PutString("TYPE")=="PSD"):
                    for j in range( dat(i).PutSize() ):
                        ec = dat(i,j)
                        sa = ec.PutHeaderPointer().PutDouble("PixelSolidAngle")
                        if sa==0.0:
                            raise UserWarning("Solid angle = 0 at %d,%d"%(i,j))
                        else:
                            ec.MulMySelf(1.0/sa)

            p_chk = mu.UtsusemiCheckDataProcess()
            p_chk.AddProcess( dat, "SolidAngleCorrection" )
            del p_chk

    return dat_list

def GetNeunetHistSASWitoutCheckPulseId(runNo="197",HistParam="tof,0,40000,100",DetParam="psd",DetRange="All",TimeRange="All",FrameInfo="None",MaskInfo="NoFile",CaseInfo="None",BGInfo="TimeDep:None",TofShift="None",AnaMode="0",ParamFiles="-,-",SolAngCor=False):
    """
    Convert Event data of NEUNET to Histogram
    GetNeunetHist(runNo="197,198",
                  HistParam="tof,0,40000,100",
                  DetParam="psd",
                  DetRange="All",
                  TimeRange="All",
                  FrameInfo="None",
                  MaskInfo="NoFile"
                  CaseInfo="None",
                  BGInfo="TimeDep:None",
                  TofShift="None",
                  AnaMode="0",
                  ParamFiles="-,-",
                  SolAngCor=False
                  )

    -----------------------
    HistParam format
        TOF
           delta-TOF const :  tof,<min_tof>,<max_tof>,<delta_tof>
           d-TOF/TOF const : rtof,<min_tof>,<max_tof>,<racio>

        TOF with time forcusing
           delta-TOF const : tf-tof,<min_tof>,<max_tof>,<delta_tof>
           d-TOF/TOF const : tf-rtof,<min_tof>,<max_tof>,<racio>

        Lambda
           delta-lambda const        :  lambda,<min_lambda>,<max_lambda>,<delta_lambda>
           delta-lambda/lambda const : rlambda,<min_lambda>,<max_lambda>,<racio>

        Energy            : energy,<min_energy>,<max_energy>,<delta_energy>

        Momentum Transfer : q,<min_Q>,<max_Q>,<delta_Q>[,<ki_x>,<ki_y>,<ki_z>]

        Energy Transfer   : hw,<Ei>,<min_hw>,<max_hw>,<delta_hw>[,<t0_shift>]

        d-value           : d,<min_d-val>,<max_d-val>,<delta_d-val>

    -----------------------
    DetParam format
        "<detType>[[<LLD_min>:<LLD_max>]][,..]" : detType in {"PSD","MON","PSD-PH","MON-PH", "UPSD", "TRIGNET"}
                                                "-PH" means that Pulse Height is calculated at the same time
    -----------------------
    DetRange format
        "All[:<number_of_pixels>]"                     : use all detectors
        "<startDetId>-<endDetId>[:<number_of_pixels>]" : use detectors from <startDetId> to <endDetId>
                                                         "X-Y:Z,A-B:C,D-E:F..." can be used for more settings
    -----------------------
    TofShift format
        "<ptnId>[,<p1>,<p2>,...]"
             ptnId = 0 : no use in this facade( only use if scattered data of tof origin shift vs lambda is given )
             ptnId = 1 : use shift = p1/(exp(-p2*(lambda-p3)) +p4) + p5*lambda
                         default p1=5.0, p2=8.464, p3=2.08765, p4=1.23477, p5=5.32657
    -----------------------

    @param runNo     (string or int) Run Number(s) 123 or "123,124"
    @param HistParam (string) "type,param,param,param,..." type must be in "tof" "rtof" "lambda" "rlambda" "hw"
    @param DetParam  (string) "DetType[,LLD,HLD]" DetType must be in "psd","n2mon","psd-ph" or "n2mon-ph"
    @param DetRange  (string) "All[:number_of_pixels]" or "startPsdId-endPsdId[:number_of_pixels]"
    @param TimeRange (string) "All" or "startTime,endTime" in [second] or "YYYY/MM/DD,hh:mm:ss,YYYY/MM/DD,hh:mm:ss"
    @param FrameInfo (string) "None" or "FrameNumber, boudaryTime"
    @param MaskInfo  (string) "<maskfile> [,<TOFmask>-<TOFmask>,..]" : if no use maskfile, set "NoFile"
    @param CaseInfo  (string) CaseInfo file must be in ${UTSUSEMI_USR_DIR}/ana/xml or ${UTSUSEMI_HOME_DIR}/ana/xml
    @param BGInfo    (string) "TimeDep:None" TimeDep:tof1-tof2
    @param TofShift  (string) "None" or "<ptnId> [,<p1>,<p2>,...]"
    @param AnaMode   (string or int) change analysis mode, which means to swich WiringInfo and DetectorInfo to be used for histogram.
    @param ParamFiles(string) "<path/to/WiringInfo>,<path/to/DetectorInfo>"
    @param SolAngCor (bool) True for doing SolidAngle Correction
    @retval DAT (ElementContainerMatrix)
    """

    HBC.CheckT0bUpdate(int(runNo))
    AnaMode = AnaMode+","+ParamFiles
    dat_list = BaseCom.GetNeunetHist(runNo,HistParam,DetParam,DetRange,TimeRange,FrameInfo,MaskInfo,CaseInfo,BGInfo,TofShift,AnaMode,Options="ISCHECKPULSEID:false")
    if type(dat_list)!=list:
        dat_list=[dat_list]
    for dat in dat_list:
        ## Create Lambda Vector for each EC
        tt1 = ms.CreateLambVector( dat )
        tt1.Execute()
        del tt1

        ## Convertion from histogram to per_unit
        tt2 = ms.ConvertToPoint(dat)
        tt2.Execute()
        del tt2

        ## Solid angle correction
        if (SolAngCor):
            for i in range(dat.PutSize()):
                if (dat(i).PutHeaderPointer().PutString("TYPE")=="PSD"):
                    for j in range( dat(i).PutSize() ):
                        ec = dat(i,j)
                        sa = ec.PutHeaderPointer().PutDouble("PixelSolidAngle")
                        if sa==0.0:
                            raise UserWarning("Solid angle = 0 at %d,%d"%(i,j))
                        else:
                            ec.MulMySelf(1.0/sa)

            p_chk = mu.UtsusemiCheckDataProcess()
            p_chk.AddProcess( dat, "SolidAngleCorrection" )
            del p_chk

    return dat_list

########################################
def DataReductionMulti( dat_list=DAT,Xrange="4.0 7.7 0.1 dL", kickerNorm=True, incVec="0.0 0.0 1.0", maskFile="mask_160316.xml" ):
    """
    Do the data reduction process as below:
      - EC = Hist.GetMonHistSAS(runNo=runno,MonNo=1,AxType=conv_type,Xrange=conv_params,MonIndex=0,TimeSlice="-1 -1",frameBoundary=0.0,CalibEffi=True)
      - Corr.I0LambdaCorrection(dat=DATs,I0Lamb=EC)
      - QSpace.ConvertQSpace(dat=DATs,InVec=incVec)
      - DoMask(dat=DATs,filename=maskFile)
      - DATs.MulMySelf( float(all_kickers)/float(kickers) ) when "kickerNorm" is True

    @param dat_list(list)
    @param Xrange(string)
    @param kickerNorm(bool) True:do kicker normalization
    @retval None
    """
    print("#########################[inamura 251223] DataReductionMulti start")
    if type(dat_list)==list:
        data = dat_list
    else:
        data = [dat_list]
    run_st=data[0].PutHeader().PutString("RUNNUMBER")
    #runno=int(run_st)
    run_st_list=run_st.split(",")

    conv_type =""
    conv_params=""
    p_list=Xrange.split(" ")
    if len(p_list)!=4:
        raise UserWarning("param error")
    AxType=p_list[3].lower()
    if AxType=="dl":
        conv_type = "lambda"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="dl/l":
        conv_type = "lambda2"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="tof":
        conv_type = "tof"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    else:
        raise UserWarning("param error about conv type")

    UCC=mu.UtsusemiCalcContainers()
    all_kickers = 0
    EC_sum=None
    for a_run_st in run_st_list:
        runno=int(a_run_st)
        ECs = Hist.GetMonHistSAS(runNo=runno,MonNo=1,AxType=conv_type,Xrange=conv_params,MonIndex=0,TimeSlice="-1 -1",frameBoundary=0.0,CalibEffi=True)
        all_kickers += ECs.PutHeaderPointer().PutInt4("KICKERCOUNT")
        if EC_sum is None:
            EC_sum = ECs
        else:
            EC_sum += ECs
    EC_sum.PutHeaderPointer().OverWrite("KICKERCOUNT",all_kickers)
    del UCC

    lambda_range = "%g %g"%(float(p_list[0]),float(p_list[1]))

    #all_kickers = ECs.PutHeaderPointer().PutInt4("KICKERCOUNT")
    for DATs in data:
        #Corr.CorrectDetEffi(dat=DATs,filename="detectorEffiTBL.dat")
        Corr.I0LambdaCorrection(dat=DATs,I0Lamb=EC_sum)
        print("#########################[inamura 251223] ConvertQspace start")
        QSpace.ConvertQSpace(dat=DATs,InVec=incVec)
        print("#########################[inamura 251223] ConvertQspace start")
        #DoMask(dat=DATs,filename=maskFile)
        if maskFile.upper()!="NONE" and maskFile!="-" and maskFile.upper()!="NOFILE":
            DoMask(dat=DATs,filename=maskFile)
        if kickerNorm:
            kickers = DATs.PutHeaderPointer().PutInt4("KICKERCOUNT")
            DATs.MulMySelf( float(all_kickers)/float(kickers) )
    if len(data)==1:
        dat_list=data[0]

########################################
def SubtractBackMulti( dat_list=DAT, dat_BG=DAT):
    """

    @param dat_list (list of ElementContainerMatrix)
    @param dat_BG (ElementCntainerMatrix)
    @retval DATs (list of ElementContainerMatrix)
    """
    if type(dat_list)!=list:
        dat_list = [dat_list]

    import Com
    ret = []
    for a_dat in dat_list:
        ret.append( Com.CalcContainers(dat1=a_dat,dat2=dat_BG,coef1=1.0,coef2=1.0,ope="-") )
    return ret

########################################
def QzProjectionMulti(dat_list=DATList,QxRange="-0.06 0.06 0.001",QyRange="-0.06 0.06 0.001",QzRange="-0.5 0.0",LambdaRange="4.0 7.7 0.01"):
    """

    @param dat_list(list)
    @param Xrange(string)
    @retval list of ElementContainerArray
    """
    ret = []
    for dat in dat_list:
        eca = QSpace.QzProjection(dat,QxRange,QyRange,QzRange,LambdaRange)
        ret.append(eca)

    return ret

########################################
def SectorAverageByAnglesMulti( dat_list=DATList, index=0, Banks="SM16-SM14-SD-SU", Qrange="0.01 2.0 0.001 dQ/Q", LamRange="0.7 0.8", AzimAng="0.0 0.0", PolAng="0.0 0.0", QLamFile="None/QLambdaRange" ):
    """
    Convert Data to Q space EventData to produce histogram
    SectorAverageByAngles( dat=DAT, Banks="SM16-SM14-SD-SU", Qrange="0.01 2.0 0.001 dq/Q", LamRange="0.7 0.8", AzimAng="0.0 0.0", PolAng="0.0 0.0" )

    @param  dat_list (list of ElementContainerMatrix)
    @param  index    (int)    index number in dat_list to analyze
    @param  Banks    (string)  "SM16-SD-SU" means these banks are unified in one Iq, "SM SD SU" means separated Iq
    @param  Qrange   (string)  Q range and type of Qbin "Qmin Qmax deltaQ type" type must be dQ/Q or dQ
    @param  LamRange (string)  Lambda range "Lambda_min Lambda_max"
    @param  AzimAng  (string)  "-20.0 20 170 -170"
    @param  PolAng   (string)  "0 90 0 40"
    @param  QLamFile (string)  filename of Q-Lambda range info. If None, no use. (read QLambdaRange_SM.txt for SM bank)
    @retval Iq       (list of ElementContainer)
    """

    return QBC.SectorAverageByAngles( dat_list[index], Banks, Qrange, LamRange, AzimAng, PolAng, QLamFile )

########################################
def GetAzimuthProfile(dat=DAT, Qcenter=0.036, dQ=0.002, azim_bin=1.0, azim_range="0.0 360.0", lambda_range="0 0"):
    """
    Calculates Azimuth Profile from DataReduction

    @param dat          (ElementContainerMatrix)
    @param Qcenter      (float)
    @param dQ           (float)
    @param azim_bin     (float)
    @param azim_range   (string)
    @param lambda_range (string)
    @retval Iap
    """
    ec_list=GetAzimuthProfileMulti([dat], Qcenter, dQ, azim_bin, azim_range, lambda_range)
    return ec_list[0]

########################################
def GetAzimuthProfileMulti(dat_list=DATList, Qcenter=0.036, dQ=0.002, azim_bin=1.0, azim_range="0.0 360.0", lambda_range="0 0"):
    """
    Calculates Azimuth Profile from DataReduction

    @param dat_list     (list)
    @param Qcenter      (float)
    @param dQ           (float)
    @param azim_bin     (float)
    @param azim_range   (string)
    @param lambda_range (string)
    @retval IapList
    """
    return QBC.GetAzimuthProfileMulti(dat_list, Qcenter, dQ, azim_bin, azim_range, lambda_range)

########################################
def Test( dat_list=DAT, Xrange="0.7 7.6 0.1 dL" ):
    """

    @param dat_list(list)
    @param Xrange(string)
    @retval list of ElementContainerArray
    """

    run_st=dat_list[0].PutHeader().PutString("RUNNUMBER")
    runno=int(run_st)

    conv_type =""
    conv_params=""
    p_list=Xrange.split(" ")
    if len(p_list)!=4:
        raise UserWarning("param error")
    AxType=p_list[3].lower()
    if AxType=="dl":
        conv_type = "lambda"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="dl/l":
        conv_type = "lambda2"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="tof":
        conv_type = "tof"
        conv_params="%g %g %g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    else:
        raise UserWarning("param error about conv type")

    ECs = Hist.GetMonHistSAS(runNo=runno,MonNo=1,AxType=conv_type,Xrange=conv_params,MonIndex=0,TimeSlice="-1 -1",frameBoundary=0.0,CalibEffi=True)

    lambda_range = "%g %g"%(float(p_list[0]),float(p_list[1]))
    ret = []
    for DATs in dat_list:
        Corr.CorrectDetEffi(dat=DATs,filename="detectorEffiTBL.dat")
        Corr.I0LambdaCorrection(dat=DATs,I0Lamb=ECs)
        QSpace.ConvertQSpace(dat=DATs,InVec="0.0 0.0 1.0")
        DoMask(dat=DATs,filename="mask_160316.xml")
        ECA = QSpace.QzProjection(dat=DATs,QxRange="-0.06 0.06 0.001",QyRange="-0.06 0.06 0.001",QzRange="-0.5 0.0",LambdaRange=lambda_range)
        ret.append( ECA )
    return ret

########################################
def PickDataByCaseId( dat=DAT, caseId=1 ):
    """
    """
    import Manyo
    if isinstance( dat, Manyo.ElementContainerMatrix() ):
        return dat
    for a_dat in dat:
        hh=a_dat.PutHeaderPointer()
        if caseId==hh.PutInt4("CASEID"):
            return a_dat

    raise UserWarning("This caseId is NOT included.")

#########################################
def DoMask(dat=DAT,filename="mask.txt"):
    """
    MASKED flag controled by file.
    @param dat [Def:DAT] (ElementContainerMatrix)
    @param filename (string) filename of mask information
    """

    BaseCom.DoMask(dat,filename)

########################################
def RemoveAllMask( dat=DAT ):
    """
    Does remove mask
    REmoveAllMask( dat, filename )

    @param dat       (ElementContainerMatrix)
    @param filename  (string) mask file must be in UTSUSEMI_USR_DIR/ana/xml
    @retval None
    """

    SMSK = mu.UtsusemiSetMask( dat )
    SMSK.UmaskAll()
    del SMSK

############################################
def LoadDataFromManyoBinary(path="./",filename="Sample.mdb"):
    """
    Load ElementContainerMatrix from manyo binary data file
    which is the binary format for storing ElementContainers usign HDF5
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval  data (ElementContaienr, -Array, -Matrix)
    """
    return BaseCom.LoadDataFromManyoBinary(path,filename)

############################################
def SaveDataToManyoBinary(dat=DAT, path="./",filename="Sample.mdb"):
    """
    Save ElementContainerMatrix to manyo binary data file
    This is the binary format for storing ElementContainers using HDF5
    @param  dat      (ElementContainer, -Array, -Matrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval None
    """
    BaseCom.SaveDataToManyoBinary(dat, path,filename)

#########################################
def LoadDataFromDump(path="./",filename="Sample.dmp"):
    """
    Load ElementContainerMatrix from dump file made by ECMSerialize class
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval DAT        ElementContainer/Array/Matrix
    """

    return BaseCom.LoadDataFromDump(path,filename)

#########################################
def SaveDataToDump(dat=DAT, path="./",filename="Sample.dmp"):
    """
    Serialize ElementContainerMatrix to dump file using ECMSerialize class.
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval None
    """

    BaseCom.SaveDataToDump(dat,path,filename)

#########################################
def GetDataFromNeXus(path="./",filename="Sample.nx"):
    """
    Load ElementContainer/Array/Matrix from NeXus file.
    @param  path     (String)   path to data file
    @param  filename (String)  data file name
    @retval DAT      ElementContainer/Array/Matrix
    """
    return BaseCom.GetDataFromNeXus(path,filename)

#########################################
def SaveDataToNeXus(dat=DAT, name="Data",path="./",filename="Sample.nx"):
    """
    Save ElementContainer/Array/Matrix to NeXus file.
    @param  dat       (ElementContainerMatrix)
    @param  name      (String)   Title of saved data
    @param  path      (String)   path to data file
    @param  filename  (String)  data file name
    @retval None
    """
    BaseCom.SaveDataToNeXus(dat, name,path,filename)

##########################################
def getBeamCurrent(start_day="2009/5/30", start_time="10:00:00", end_day="2009/5/30", end_time="11:00:00" ):
    """
    get Beam current from web and calculate summation of the number of shots and protons
    @param start_day  the day of measurement start
    @param start_time the time of measurement start
    @param end_day    the day of measurement end
    @param end_time   the time of measurement end
    @retval ret       the number of protons [TelaProtons]
    """
    return BaseCom.getBeamCurrent(start_day, start_time, end_day, end_time)

##########################################
def NormByBeamCurrent(dat=DAT,factor=1000.0 ):
    """
    Normailze by Beam current from web and calculate summation of the number of shots and protons. Experimental info file, like SIK000XXX.xml file, must be placed in your ~/exp/xml directory.
    @param dat        (ElementContainer,-Array,-Matrix)
    @param factor     (double) normalize factor data*factor/beamCurrent
    """
    BaseCom.NormByBeamCurrent(dat, factor)



#########################################
def PrintObject(target=PyObj):
    """
    Print object for debug.
    @param target print to stdout
    @retval None
    """
    print(type(target))
    try:
        print(target)
    except:
        print("not printable")

#########################################
def MergeDataECM(dat1=DAT, coef1=1.0, dat2=DAT, coef2=0.0, dat3=DAT, coef3=0.0, dat4=DAT, coef4=0.0):
    """
    Merge Data (ElementContainerMatrix only)
    @param dat1  (ElementContainerMatrix)
    @param coef1 (float) coefficient for dat1
    @param dat2  (ElementContainerMatrix)
    @param coef2 (float) coefficient for dat2
    @param dat3  (ElementContainerMatrix)
    @param coef3 (float) coefficient for dat3
    @param dat4  (ElementContainerMatrix)
    @param coef4 (float) coefficient for dat4
    @retval DATmerg  (ElementContainerMatrix)
    """
    if not isinstance( dat1, Manyo.ElementContainerMatrix ):
        raise UserWarning("dat1 is not ElementContainerMatrix")
    if not isinstance( dat2, Manyo.ElementContainerMatrix ):
        raise UserWarning("dat2 is not ElementContainerMatrix")
    if not isinstance( dat3, Manyo.ElementContainerMatrix ):
        raise UserWarning("dat3 is not ElementContainerMatrix")
    if not isinstance( dat4, Manyo.ElementContainerMatrix ):
        raise UserWarning("dat4 is not ElementContainerMatrix")

    ret = dat1.Mul( coef1 )
    hh = ret.PutHeaderPointer()
    if (coef2!=0.0):
        for i in range( dat2.PutSize() ):
            ret.Add( dat2.PutPointer(i).Mul(coef2) )
        hht = dat2.PutHeaderPointer()
        if (hh.CheckKey("BANKIDLIST")==1) and (hht.CheckKey("BANKIDLIST")==1):
            bidvect=hh.PutInt4Vector("BANKIDLIST")
            bszvect=hh.PutInt4Vector("BANKSIZELIST")
            bnmvect=hh.PutStringVector("BANKNAMELIST")
            bidvectt=hht.PutInt4Vector("BANKIDLIST")
            bszvectt=hht.PutInt4Vector("BANKSIZELIST")
            bnmvectt=hht.PutStringVector("BANKNAMELIST")
            for i in range( bidvectt.size() ):
                bidvect.push_back( bidvectt[i] )
                bszvect.push_back( bszvectt[i] )
                bnmvect.push_back( bnmvectt[i] )
            hh.OverWrite("BANKIDLIST",bidvect)
            hh.OverWrite("BANKSIZELIST",bszvect)
            hh.OverWrite("BANKNAMELIST",bnmvect)

    if (coef3!=0.0):
        for i in range( dat3.PutSize() ):
            ret.Add( dat3.PutPointer(i).Mul(coef3) )
        hht = dat3.PutHeaderPointer()
        if (hh.CheckKey("BANKIDLIST")==1) and (hht.CheckKey("BANKIDLIST")==1):
            bidvect=hh.PutInt4Vector("BANKIDLIST")
            bszvect=hh.PutInt4Vector("BANKSIZELIST")
            bnmvect=hh.PutStringVector("BANKNAMELIST")
            bidvectt=hht.PutInt4Vector("BANKIDLIST")
            bszvectt=hht.PutInt4Vector("BANKSIZELIST")
            bnmvectt=hht.PutStringVector("BANKNAMELIST")
            for i in range( bidvectt.size() ):
                bidvect.push_back( bidvectt[i] )
                bszvect.push_back( bszvectt[i] )
                bnmvect.push_back( bnmvectt[i] )
            hh.OverWrite("BANKIDLIST",bidvect)
            hh.OverWrite("BANKSIZELIST",bszvect)
            hh.OverWrite("BANKNAMELIST",bnmvect)


    if (coef4!=0.0):
        for i in range( dat4.PutSize() ):
            ret.Add( dat4.PutPointer(i).Mul(coef4) )
        hht = dat4.PutHeaderPointer()
        if (hh.CheckKey("BANKIDLIST")==1) and (hht.CheckKey("BANKIDLIST")==1):
            bidvect=hh.PutInt4Vector("BANKIDLIST")
            bszvect=hh.PutInt4Vector("BANKSIZELIST")
            bnmvect=hh.PutStringVector("BANKNAMELIST")
            bidvectt=hht.PutInt4Vector("BANKIDLIST")
            bszvectt=hht.PutInt4Vector("BANKSIZELIST")
            bnmvectt=hht.PutStringVector("BANKNAMELIST")
            for i in range( bidvectt.size() ):
                bidvect.push_back( bidvectt[i] )
                bszvect.push_back( bszvectt[i] )
                bnmvect.push_back( bnmvectt[i] )
            hh.OverWrite("BANKIDLIST",bidvect)
            hh.OverWrite("BANKSIZELIST",bszvect)
            hh.OverWrite("BANKNAMELIST",bnmvect)

    return ret

########################################
def CopyData(first_run=10,last_run=-1,flag=0):
    """
    Copy EventData file from DAQ0 or DAQ1 into /data/XXX
    @param runno_from (int) First Run No of a measurement to be copied
    @param runno_to (int) Last Run No of a measurement to be copied
    @param flag  (int)  Flag for Copy(0) or Symbolic link(1) / default is Symbolic link or -1 for testing
    @retval None
    """
    BaseCom.CopyData( first_run, last_run, flag )

#########################################
def GetPH(dat=DAT, binSize=1):
    """
    Get Pulse Height Profile of PSD
    Each PSD pulse heignt profile is stored in header of ElementContaienrArray.
    @param  dat   (ElementContainerMatrix) Loaded ElementContainerMatrix.
    @param  binSize (UInt4)
    @retval ECS    ElementContainer or list of ElementContainers of pulse height profile
    """
    if type(dat)==list:
        import Manyo
        eca = Manyo.ElementContainerArray(dat[0].PutHeader())
        for a_dat in dat:
            eca_tmp = BaseCom.GetPH(a_dat,binSize)
            for i in range( eca_tmp.PutSize() ):
                eca.Add(eca_tmp.Put(i))
        return eca
    else:
        return BaseCom.GetPH(dat,binSize)

#########################################
def DetectorEffi(dat=DAT, dataFile="SUS304-3He-PSD-SAS-6atm.dat"):
    """
    DetectorEfficiency correction by using Manyo default functions
    @param dat [Def:DAT] (ElementContainerMatrix)
    @param dataFile (string)
    @return None
    """
    import Manyo.SAS as ms
    tt = ms.SASDetectorEfficiencyCorrection()
    if tt.LoadDataFile( dataFile ):
        tt.Execute( dat )
    else:
        raise UserWarning("Invalid Parameters")

#########################################
def SetRPMTMask2thetaByGrav(dat=DAT, Angle=0.25, isOutSide=True):
    """
    set masked-status to pixels which 2theta is inside of given angle with the center positions dropping by a gravity
    @param dat [Def:DAT]
    @param Angle   [deg]
    @param isOutSide (bool) set mask outside of given angle or not
    @return None
    """
    PP=ms.RPMTSetMaskBy2theta(dat, Angle, isOutSide)
    if PP.Status():
        PP.Execute()
    return
    """
    modeNo = 5
    runNo_st = dat.PutHeader().PutString(mu.UTSUSEMI_KEY_HEAD_RUNNUMBER)
    if len(runNo_st)==6:
        runNo = int(runNo_st)
    elif len(runNo_st)==9:
        runNo = int(runNo_st[3:])

    ANE = mu.UtsusemiAnaEnvironReader("environ_ana.xml")
    v_pfiles = ANE.PutParamFiles(runNo, modeNo, True)
    if v_pfiles.size() == 0:
        UserWarning("RunNo is invalid.")

    SDE = ms.SASDetectorInfoEditorRPMT(v_pfiles[1], True);
    b = SDE.PutIncidentGravityParam()
    v0 = dat(0,0).PutHeader().PutDoubleVector(mu.UTSUSEMI_KEY_HEAD_PIXELPOSITION)
    r_ang = v0[2] * math.tan(Angle / 180.0 * math.pi)
    ekey = dat(0,0).PutEKey()
    for i in range(dat.PutSize()):
        eca = dat(i)
        hh = eca.PutHeaderPointer()
        if hh.CheckKey(mu.UTSUSEMI_KEY_HEAD_MASKED) == 0:
            isMasked = False
        elif hh.PutInt4(mu.UTSUSEMI_KEY_HEAD_MASKED) == 1:
            isMasked = True
        else:
            isMasked = False
        if isMasked:
            continue

        cnt_ec = 0
        for j in range(eca.PutSize()):
            ec = eca(j)
            hh_ec = ec.PutHeaderPointer()
            if (hh_ec.CheckKey(mu.UTSUSEMI_KEY_HEAD_MASKED) == 0) or (hh.PutInt4(mu.UTSUSEMI_KEY_HEAD_MASKED) == 0):
                errv = ec.PutP(ekey)
                lamv = ec.Put("Lamb")
                pv = hh_ec.PutDoubleVector(mu.UTSUSEMI_KEY_HEAD_PIXELPOSITION)
                cnt_lam = 0
                for k in range(lamv.size()):
                    setMask = False
                    cy = -1.0 * b * lamv[k] * lamv[k]
                    d = math.sqrt((pv[0] * pv[0]) + (pv[1] - cy) * (pv[1] - cy) )
                    if isOutSide:
                        if d > r_ang:
                            setMask = True
                    else:
                        if d <= r_ang:
                            setMask = True
                    if setMask:
                        errv[k] = -1.0 * errv[k]
                        cnt_lam += 1
                if cnt_lam == lamv.size():
                    hh_ec.OverWrite(mu.UTSUSEMI_KEY_HEAD_MASKED, 1)
                    cnt_ec += 1

        if cnt_ec == eca.PutSize():
            hh.OverWrite(mu.UTSUSEMI_KEY_HEAD_MASKED, 1)
    """
#########################################
def SetRPMTMaskBy2theta(dat=DAT, detId=154, Pixel=140, Angle=0.25, isOutSide=True):
    """
    set masked-status to pixels which 2theta is inside of given angle
    @param dat [Def:DAT]
    @param detId
    @param Pixel
    @param Angle   [deg]
    @param isOutSide (bool)
    @return None
    """
    import Manyo
    import math
    SIH = Manyo.SearchInHeader()
    SIH.SetTarget(dat)
    SIH.SearchArray( "DETID", detId )
    ind_v = SIH.PutResultIndex(0)
    del SIH
    if ind_v.size()==0:
        raise UserWarning("Such detId is not found")

    hh_org=dat(ind_v[0]).Put(Pixel).PutHeader()
    p_org = hh_org.PutDoubleVector("PixelPosition")
    po_x = p_org[0]
    po_y = p_org[1]
    po_z = p_org[2]
    lo = math.sqrt( po_x*po_x + po_y*po_y + po_z*po_z )
    po_x = po_x/lo
    po_y = po_y/lo
    po_z = po_z/lo

    for i in range( dat.PutSize() ):
        eca = dat(i)
        cnt = 0
        for j in range( eca.PutSize() ):
            if i==ind_v[0] and j==Pixel:
                continue
            ec = dat(i,j)
            hh = ec.PutHeaderPointer()
            p_v=hh.PutDoubleVector("PixelPosition")
            p_x = p_v[0]
            p_y = p_v[1]
            p_z = p_v[2]
            lp = math.sqrt( p_x*p_x + p_y*p_y + p_z*p_z )
            p_x = p_x/lp
            p_y = p_y/lp
            p_z = p_z/lp
            theta_rad = math.acos( po_x*p_x + po_y*p_y + po_z*p_z )
            theta_deg = theta_rad*180.0/math.pi
            if isOutSide:
                if theta_deg > Angle:
                    hh.OverWrite("MASKED",1)
                    cnt += 1
            else:
                if theta_deg <= Angle:
                    hh.OverWrite("MASKED",1)
                    cnt += 1
        #print( "Masked count=%d"%cnt)
        if cnt==eca.PutSize():
            eca.PutHeaderPointer().OverWrite("MASKED",1)

#########################################
def IncohCalcFor3HeNSF(DAT1=DAT, DAT2=DAT, A=0.0, PHe=0.0, PolData="BL15_pol_vs_lambda.dat", is2ThetaCorr=True):
    """
    Separate Coherent and Incoherent components from given data set measured with 3He spin flipper.

    Return value is given as list with size of 2. To pick up each component data, use Cmm.ExecFunctionCode like below:
    CohDat = Cmm.ExecFunctionCode(Code="A[0]", A=CohInc, B=CohInc )
    IncDat = Cmm.ExecFunctionCode(Code="A[1]", A=CohInc, B=CohInc )

    @param DAT1 [Def:DAT1] (ElementContainerMatrix) Data for spin flipper on
    @param DAT2 [Def:DAT2] (ElementContainerMatrix) Data for spin flipper off
    @param A    (Double) A(lambda) value
    @param PHe  (Double) the pressure of 3He
    @param PolData (str) file path to text data for the polarization vs lambda
    @param is2ThetaCorr (bool) use A/cos(2theta)*lambda insteed of A*lambda
    @retval CohInc     list of coherent and incoherent components from given data [<Coh>, <Inc>]
    """
    IC=ms.IncohCalcFor3HeSpinFilter( DAT1, DAT2, A, PHe, is2ThetaCorr )
    if not IC.SetPolarizationTable( PolData ):
        raise UserWarning("Failed to load Polorization data.")

    DATc = Manyo.ElementContainerMatrix()
    DATic = Manyo.ElementContainerMatrix()
    if not IC.Execute(DATc, DATic):
        del IC
        raise UserWarning("Failed to execute")
    del IC
    return [DATc, DATic]

#########################################
def IncohCalcFor3HeNSFmul(DAT1=DAT, DAT2=DAT, A=0.0, PHe_p=0.0, PHe_m=0.0, Pmul=0.6, PolData="BL15_pol_vs_lambda.dat", is2ThetaCorr=True):
    """
    Separate Coherent and Incoherent components from given data set measured with 3He spin flipper (with coefficient for multi-scattering)

    Return value is given as list with size of 2. To pick up each component data, use Cmm.ExecFunctionCode like below:
    CohDat = Cmm.ExecFunctionCode(Code="A[0]", A=CohInc, B=CohInc )
    IncDat = Cmm.ExecFunctionCode(Code="A[1]", A=CohInc, B=CohInc )
    NSFDat = Cmm.ExecFunctionCode(Code="A[2]", A=CohInc, B=CohInc )
    SFDat = Cmm.ExecFunctionCode(Code="A[3]", A=CohInc, B=CohInc )

    @param DAT1 [Def:DAT1] (ElementContainerMatrix) Data for spin flipper on
    @param DAT2 [Def:DAT2] (ElementContainerMatrix) Data for spin flipper off
    @param A    (Double) A(lambda) value
    @param PHe_p  (Double) the pressure of 3He
    @param PHe_m  (Double) the pressure of 3He
    @param Pmul   (Double) the coefficient value for multi-scattering
    @param PolData (str) file path to text data for the polarization vs lambda
    @param is2ThetaCorr (bool) use A/cos(2theta)*lambda insteed of A*lambda
    @retval CohInc     list of coherent and incoherent components from given data [<Coh>, <Inc>]
    """
    IC=ms.IncohCalcFor3HeSpinFilter( DAT1, DAT2, A, PHe_p, PHe_m, is2ThetaCorr )
    if not IC.SetPolarizationTable( PolData ):
        raise UserWarning("Failed to load Polorization data.")

    DATc = Manyo.ElementContainerMatrix()
    DATic = Manyo.ElementContainerMatrix()
    DATnsf = Manyo.ElementContainerMatrix() # non-spin-flip
    DATsf = Manyo.ElementContainerMatrix()  # spin-flip

    if IC.SetMultiScatPval(Pmul):
        if not IC.Execute(DATc, DATic, DATnsf, DATsf):
            del IC
            raise UserWarning("Failed to execute")
    del IC
    return [DATc, DATic, DATnsf, DATsf]

#########################################
def IntegPixels(Target=DAT, PSD_min=1, PSD_max=1, Pixel_min=1, Pixel_max=1, isAve=True):
    """
    Put summation of intensity along Pixel
    @param Target [Def:DAT]   (ElementContainerMatrix)
    @param PSD_min   (int) lower limit of PSD-ID range
    @param PSD_max   (int) upper limit of PSD-ID range
    @param Pixel_min (int) lower limit of Pixel range
    @param Pixel_max (int) upper limit of Pixel range
    @param isAve     (bool) averaging or summation
    @retval EC
    """
    return BaseCom.IntegPixels(Target, PSD_min, PSD_max, Pixel_min, Pixel_max, isAve)

#########################################
# Dictionary for entry the name of functions
_functions = {
              "GetNeunetHistSAS":GetNeunetHistSAS,
              "GetNeunetHistSASWitoutCheckPulseId":GetNeunetHistSASWitoutCheckPulseId,
              "DataReductionMulti":DataReductionMulti,
              "SubtractBackMulti":SubtractBackMulti,
              "QzProjectionMulti":QzProjectionMulti,
              "GetAzimuthProfile":GetAzimuthProfile,
              "GetAzimuthProfileMulti":GetAzimuthProfileMulti,
              "IncohCalcFor3HeNSF":IncohCalcFor3HeNSF,
              "IncohCalcFor3HeNSFmul":IncohCalcFor3HeNSFmul,
              "SectorAverageByAnglesMulti":SectorAverageByAnglesMulti,
              "IntegPixels":IntegPixels,
              "Test":Test,
              "NormByBeamCurrent":NormByBeamCurrent,
              "getBeamCurrent":getBeamCurrent,
              "SaveDataToManyoBinary":SaveDataToManyoBinary,
              "LoadDataFromManyoBinary":LoadDataFromManyoBinary,
              "SaveDataToDump":SaveDataToDump,
              "LoadDataFromDump":LoadDataFromDump,
              "SaveDataToNeXus":SaveDataToNeXus,
              "GetDataFromNeXus":GetDataFromNeXus,
              "PrintObject":PrintObject,
              "DoMask":DoMask,
              "RemoveAllMask":RemoveAllMask,
              "MergeDataECM":MergeDataECM,
              "CopyData":CopyData,
              "GetPH":GetPH,
              "DetectorEffi":DetectorEffi,
              "SetRPMTMask2thetaByGrav":SetRPMTMask2thetaByGrav,
              "SetRPMTMaskBy2theta":SetRPMTMaskBy2theta
	      }

_functionsOrder = [
              "GetNeunetHistSAS",
              "DataReductionMulti",
              "SubtractBackMulti",
              "QzProjectionMulti",
              "GetAzimuthProfileMulti",
              "GetAzimuthProfile",
              "SectorAverageByAnglesMulti",
              "IntegPixels",
              "IncohCalcFor3HeNSF",
              "IncohCalcFor3HeNSFmul",
              "Test",
              "NormByBeamCurrent",
              "getBeamCurrent",
              "SaveDataToManyoBinary",
              "LoadDataFromManyoBinary",
              "SaveDataToDump",
              "LoadDataFromDump",
              "SaveDataToNeXus",
              "GetDataFromNeXus",
              "PrintObject",
              "DoMask",
              "RemoveAllMask",
              "MergeDataECM",
              "CopyData",
              "GetPH",
              "DetectorEffi",
              "SetRPMTMask2thetaByGrav",
              "SetRPMTMaskBy2theta"
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
#import time,sys,os,math,glob,datetime,re,urllib,httplib,socket
import time,sys,os,math,glob,datetime,re,urllib,socket
import Manyo as mm
import Manyo.Utsusemi as mu
import Manyo.DNA as md

import utsusemi.ana.Reduction.BaseCommandsInEla as BaseCom
import utsusemi.DNA.ana.Reduction.PeakFit as PF
import shutil
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

from utsusemi.DNA.ana.Reduction.DefaultInstParams import *
socket.setdefaulttimeout(1)
#------------------------------------------------------------------------------------------------------------

#------------------------------------------------------------------------------------------------------------
def CheckSampleTempByRunNumber(runNo,temptype): 
        """ 
         Check Sample Temperature by using LakeShore-350 recorded by IROHA2 Device method. 
         @param runNo      (int) 
         @param temptype   (string) 
         @retval EC        (ElementContainer) 
 
        """ 
        TimeParam="-1.0,-1.0" 
        
        #from DNA.ana.Reduction.BaseCommandsDNA import CheckRealTime
        #startday,starttime,endday,endtime = CheckRealTime(runNo,TimeParam) 
        
        #import DNA.ana.Reduction.TempMonitor as TM 
        resultEC = CheckSampleTemp(runNo,temptype,True) 
        resultEC.AddToHeader("RUNNUMBER",str(runNo)) 
        return resultEC
#------------------------------------------------------------------------------------------------------------
def CheckSampleTemp(runNo=9949,tempType="D2",ECOutPut=True): 
     """
         Check Sample Temperature by using LakeShore-350 and IROHA2
         @param runNo      (string) runNo
         @param thermoType   (string) select temperature channel (A, B, C, D, D2, D3, D4, D5)
         @param ECOutPut   (bool) output EC (True) or Meanvalue (False)
         @retval result    (array)
     """

     funcName = "CheckSampleTemp"
     #Check temperature sensor
     thermoType = GetthermoType(tempType) 
     
     #Check file
     #basepath = os.path.join(os.environ["UTSUSEMI_DATA_DIR"], os.environ["UTSUSEMI_SYS_NAME"])
     basepath = mu.UtsusemiEnvGetInstDataDir()

     #searchpath = "%s%06d_????????" %(os.environ["UTSUSEMI_SYS_NAME"], int(runNo))
     searchpath = "%s%06d_????????" %(mu.UtsusemiEnvGetInstCode(), int(runNo))
     result = glob.glob(os.path.join(basepath,searchpath))
     print(basepath)
     print(searchpath)
     print(result)
     
     if ("TC" in tempType):
          filepath = "Logs/LS350HTStickWithTC_Loop1%06d.log" %(int(runNo))
     elif ("PEACE_U" in tempType):
          filepath = "Logs/LS350PEACE_U_Loop1%06d.log" %(int(runNo))
          if ("UB" in tempType):
              filepath = "Logs/LS350IVCBottomLoop1%06d.log" %(int(runNo))
     elif ("PEACE_M" in tempType):
          filepath = "Logs/LS350PEACE_M_Loop3%06d.log" %(int(runNo))
          if ("MB" in tempType):
              filepath = "Logs/LS350IVCBottomLoop1%06d.log" %(int(runNo))
     elif ("PEACE_L" in tempType):
          filepath = "Logs/LS350PEACE_L_Loop4%06d.log" %(int(runNo))
          if ("LB" in tempType):
              filepath = "Logs/LS350IVCBottomLoop1%06d.log" %(int(runNo))
     else:
          filepath = "Logs/LS350Loop1%06d.log" %(int(runNo))
     print(filepath)
     if len(result) > 0:
          if os.path.exists(os.path.join(result[0],filepath)):
              filepath = os.path.join(result[0],filepath)
          else:
              print("%s >>> is %s does not exists." %(funcName, filepath))
    
     print("%s >>> filepath is %s." %(funcName, filepath))

     ####################################################################################################

     #Open file
     f = open(filepath)
     lines = f.readlines()
     f.close()
     line_num = len(lines)

     start_num=0
     end_num=0

     dt = datetime.datetime
     start_time_obj = dt.strptime(lines[0].split(",")[0],"%Y-%m-%d %H:%M:%S")

     time_array = []
     temperatures = []
     elist = []

     for line in lines:
          infos = line.split(",")
          #if len(infos) >= 12:
          if len(infos) >= 2:
               time_array.append((dt.strptime(infos[0],"%Y-%m-%d %H:%M:%S")-start_time_obj).seconds)
               temperatures.append(float(infos[thermoType]))
               elist.append(0.0)

     x1list = time_array[1:]
     x2list = time_array[:-1]
     xlist = [(x1+x2)*0.5 for x1, x2 in zip(x1list,x2list)]
     
     del x1list, x2list
     xlist.insert(0,time_array[0]-(xlist[0]))
     xlist.append(time_array[-1]+(time_array[-1]-time_array[-2])*0.5)
    
     print(xlist[0],xlist[-1],xlist[-2])

     result_ec = mm.ElementContainer()
     result_ec.Add("Xaxis",xlist,"sec")
     result_ec.Add("Yaxis",temperatures,"K")
     result_ec.Add("Eaxis",elist)
     result_ec.SetKeys("Xaxis","Yaxis","Eaxis")

#     f = open("test.csv","w")
#     for ii in range(len(xlist)):
#          f.write("%s, %s\n" %(xlist[ii],temperatures[ii]))
#     f.close()


     return result_ec
#     end_num = len(time_array)
#
#     print "start_time_h", start_time_h
#     print "end_time_h", end_time_h 
#     start_day = 2+365*12
#
#     # calclation of mean min max and so on
#
#     start_val = checkStartEndVal(list_array,start_num,param,thermoType)
#     end_val = checkStartEndVal(list_array,(end_num-1),param,thermoType)
#     min_val,max_val,sum_val,time_integral = CalcVal(list_array,start_num,end_num,param,thermoType,time_array)
#
#     # append calc result      
#
#     result = []
#     if abs(time_integral) < 0.00001:
#           print "integral of data is zero ?? please check start-time and end time"
#           print time_integral 
#           for i in range(0,20):
#                 result.append(0)
#     else:
#           result.append(float(sum_val/time_integral))
#           result.append(float(min_val))
#           result.append(float(max_val))
#           result.append(float(start_val))
#           result.append(float(end_val))
#
#     print "TemperatureRead >> Temp[K]: mean=%.2f,\tmin=%.2f,\tmax=%.2f,\tstart=%.2f,\tend=%.2f"\
#            %(result[0], result[1], result[2], result[3], result[4])
#  
#   #           result_ec = mm.ElementContainer()
#   #           result = []
#   #           return result,result_ec
#     # put value to ElementContainer
#     Xaxis = time_array
#     Yaxis = []
#     Eaxis = []
#
#     if ECOutPut :
#          if len(Xaxis)>2:
#               for i in range(len(Xaxis)-1):
#                    TM1 = checkStartEndVal(list_array,i,param,thermoType)
#                    TM2 = checkStartEndVal(list_array,i+1,param,thermoType)
#                    Yaxis.append((TM1+TM2)/2)
#                    Eaxis.append(0)
#
#               result_ec = mm.ElementContainer()
#               result_ec.Add("Xaxis",Xaxis,"sec")
#               result_ec.Add("Yaxis",Yaxis,"K")
#               result_ec.Add("Eaxis",Eaxis)
#               result_ec.SetKeys("Xaxis","Yaxis","Eaxis")
#
#              #p = mp.MPlot(result_ec)
#              #p.SetPlotParam(0,ls="-",lw=4,mk="",eb=False)
#              #p.SetYLabel("Temperature")
#              #p.SetXLabel("Time")
#              #subTitle = date1+" "+start_time+" ~ "+date2+" "+end_time
#              #p.SetSubTitle(subTitle)
#
#               return result_ec
#          else:
#               result_ec = mm.ElementContainer()
#               return result_ec
#     else:
#          return result
#     ##### -> TI & TY [20130121]
#######################################################################################

def checkStartEndVal(list_array,ii,param,thermoType):

     if(ii<len(list_array)):
          return_val = list_array[ii][param+(thermoType-1)*2]
     else:
          print("Check ii>", ii)
          reurn_val = 0

     if return_val=="": return_val1=0
     return float(return_val)

######################################################################################

def CalcVal(list_array,start_num,end_num,param,thermoType,time_array):

     sum_val = 0
     min_val = 10000
     max_val = 0
     time_integral = 0

     for i in range(start_num,end_num-1):
       if list_array[i][1] =="STATUS":

           if list_array[i][param+(thermoType-1)*2] == "":
               print("Check thermoType")              
               val = 0
           else:
               val =  float(list_array[i][param+(thermoType-1)*2])
               sum_val += val * (time_array[i+1] - time_array[i])
               time_integral += time_array[i-start_num+1] - time_array[i-start_num]
               if min_val>val:
                    min_val=val
               if max_val<val:
                    max_val=val

     return min_val,max_val,sum_val,time_integral

######################################################################################
def GetthermoType(TCh="D"):
    if(TCh=="A"):
        return 2
    elif(TCh=="B"):
        return 3
    elif(TCh=="C")or(TCh=="C_TC"):
        return 4
    elif(TCh=="D")or(TCh=="D1")or(TCh=="D_TC"):
        return 5
    elif(TCh=="D2")or(TCh=="PEACE_U"):
        return 6
    elif(TCh=="D3")or(TCh=="PEACE_M")or(TCh=="PEACE_UB"):
        return 7
    elif(TCh=="D4")or(TCh=="PEACE_L")or(TCh=="PEACE_MB"):
        return 8
    elif(TCh=="D5")or(TCh=="PEACE_LB"):
        return 9
    elif(int(TCh)>=1 and int(TCh)<=10):
        return int(TCh)
    else:
        return 4
######################################################################################


if __name__ == '__main__':
    ec = CheckSampleTemp(20740, "PEACE_U", True)
    import uGao.MPlot as mp
    mp.MPlot(ec)
